# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from botocore.exceptions import DataNotFoundError
from botocore.waiter import WaiterModel

from awscli.customizations.exceptions import ParamValidationError
from awscli.customizations.waiters import (
    WaitCommand,
    WaiterCaller,
    WaiterStateCommand,
    WaiterStateCommandBuilder,
    WaiterStateDocBuilder,
    add_waiters,
    get_waiter_model_from_service_model,
)
from awscli.testutils import (
    BaseAWSCommandParamsTest,
    BaseAWSHelpOutputTest,
    mock,
    unittest,
)


class TestAddWaiters(unittest.TestCase):
    def setUp(self):
        self.service_model = mock.Mock()
        self.session = mock.Mock()

        self.command_object = mock.Mock()
        self.command_object.service_model = self.service_model

        # Set up the mock session.
        self.session.get_waiter_model.return_value = WaiterModel(
            {
                'version': 2,
                'waiters': {
                    'FooExists': {},
                },
            }
        )

    def test_add_waiters(self):
        command_table = {}
        add_waiters(command_table, self.session, self.command_object)
        # Make sure a wait command was added.
        self.assertIn('wait', command_table)
        self.assertIsInstance(command_table['wait'], WaitCommand)

    def test_add_waiters_no_waiter_names(self):
        self.session.get_waiter_model.return_value = WaiterModel(
            {
                'version': 2,
                # No waiters are specified.
                'waiters': {},
            }
        )
        command_table = {}
        add_waiters(command_table, self.session, self.command_object)
        # Make sure that no wait command was added since the service object
        # has no waiters.
        self.assertEqual(command_table, {})

    def test_add_waiters_no_service_object(self):
        command_table = {}
        self.command_object.service_model = None
        add_waiters(command_table, self.session, self.command_object)
        # Make sure that no wait command was added since no service object
        # was passed in.
        self.assertEqual(command_table, {})

    def test_add_waiter_no_waiter_config(self):
        self.session.get_waiter_model.side_effect = DataNotFoundError(
            data_path='foo'
        )
        command_table = {}
        add_waiters(command_table, self.session, self.command_object)
        self.assertEqual(command_table, {})


class TestServicetoWaiterModel(unittest.TestCase):
    def test_service_object_to_waiter_model(self):
        service_model = mock.Mock()
        session = mock.Mock()
        service_model.service_name = 'service'
        service_model.api_version = '2014-01-01'
        get_waiter_model_from_service_model(session, service_model)
        session.get_waiter_model.assert_called_with('service')

    def test_can_handle_data_errors(self):
        service_model = mock.Mock()
        session = mock.Mock()
        service_model.service_name = 'service'
        service_model.api_version = '2014-01-01'
        session.get_waiter_model.side_effect = DataNotFoundError(
            data_path='foo'
        )
        self.assertIsNone(
            get_waiter_model_from_service_model(session, service_model)
        )


class TestWaitCommand(unittest.TestCase):
    def setUp(self):
        self.session = mock.Mock()
        self.model = WaiterModel(
            {
                'version': 2,
                'waiters': {
                    'Foo': {
                        'operation': 'foo',
                        'maxAttempts': 1,
                        'delay': 1,
                        'acceptors': [],
                    }
                },
            }
        )
        self.service_model = mock.Mock()
        self.cmd = WaitCommand(self.session, self.model, self.service_model)

    def test_passes_on_lineage(self):
        child_cmd = self.cmd.subcommand_table['foo']
        self.assertEqual(len(child_cmd.lineage), 2)
        self.assertEqual(child_cmd.lineage[0], self.cmd)
        self.assertIsInstance(child_cmd.lineage[1], WaiterStateCommand)

    def test_run_main_error(self):
        self.parsed_args = mock.Mock()
        self.parsed_args.subcommand = None
        with self.assertRaises(ParamValidationError):
            self.cmd._run_main(self.parsed_args, None)


class TestWaitHelpOutput(BaseAWSHelpOutputTest):
    def test_wait_command_is_in_list(self):
        self.driver.main(['ec2', 'help'])
        self.assert_contains('* wait')

    def test_wait_help_command(self):
        self.driver.main(['ec2', 'wait', 'help'])
        self.assert_contains('.. _cli:aws ec2 wait:')
        self.assert_contains('Wait until a particular condition is satisfied.')
        self.assert_contains('* instance-running')
        self.assert_contains('* vpc-available')

    def test_wait_state_help_command(self):
        self.driver.main(['ec2', 'wait', 'instance-running', 'help'])
        self.assert_contains('.. _cli:aws ec2 wait instance-running:')
        self.assert_contains(
            'Wait until JMESPath query Reservations[].Instances[].State.Name'
        )
        self.assert_contains('poll every')
        self.assert_contains('This will exit with a return code of 255 after')
        self.assert_contains('``describe-instances``')
        self.assert_contains('[--filters <value>]')
        self.assert_contains('``--filters`` (list)')
        self.assert_contains('======\nOutput\n======\n\nNone')


class TestWait(BaseAWSCommandParamsTest):
    """This is merely a smoke test.

    Its purpose is to test that the wait command can be run properly for
    various services. It is by no means exhaustive.
    """

    def test_ec2_instance_running(self):
        cmdline = 'ec2 wait instance-running'
        cmdline += ' --instance-ids i-12345678 i-87654321'
        cmdline += """ --filters {"Name":"group-name","Values":["foobar"]}"""
        result = {
            'Filters': [{'Name': 'group-name', 'Values': ['foobar']}],
            'InstanceIds': ['i-12345678', 'i-87654321'],
        }
        self.parsed_response = {
            'Reservations': [{'Instances': [{'State': {'Name': 'running'}}]}]
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_dynamodb_table_exists(self):
        cmdline = 'dynamodb wait table-exists'
        cmdline += ' --table-name mytable'
        result = {"TableName": "mytable"}
        self.parsed_response = {'Table': {'TableStatus': 'ACTIVE'}}
        self.assert_params_for_cmd(cmdline, result)

    def test_rds_jobs_complete(self):
        cmdline = 'rds wait db-instance-available'
        cmdline += ' --db-instance-identifier abc'
        result = {'DBInstanceIdentifier': 'abc'}
        self.parsed_response = {
            'DBInstances': [{'DBInstanceStatus': 'available'}]
        }
        self.assert_params_for_cmd(cmdline, result)


class TestWaiterStateCommandBuilder(unittest.TestCase):
    def setUp(self):
        self.session = mock.Mock()
        self.service_model = mock.Mock()

        # Create some waiters.
        self.model = WaiterModel(
            {
                'version': 2,
                'waiters': {
                    'InstanceRunning': {
                        'description': 'My waiter description.',
                        'delay': 1,
                        'maxAttempts': 10,
                        'operation': 'MyOperation',
                    },
                    'BucketExists': {
                        'description': 'My waiter description.',
                        'operation': 'MyOperation',
                        'delay': 1,
                        'maxAttempts': 10,
                    },
                },
            }
        )

        self.waiter_builder = WaiterStateCommandBuilder(
            self.session, self.model, self.service_model
        )

    def test_build_waiter_state_cmds(self):
        subcommand_table = {}
        self.waiter_builder.build_all_waiter_state_cmds(subcommand_table)
        # Check the commands are in the command table
        self.assertEqual(len(subcommand_table), 2)
        self.assertIn('instance-running', subcommand_table)
        self.assertIn('bucket-exists', subcommand_table)

        # Make sure that the correct operation object was used.
        self.service_model.operation_model.assert_called_with('MyOperation')

        # Introspect the commands in the command table
        instance_running_cmd = subcommand_table['instance-running']
        bucket_exists_cmd = subcommand_table['bucket-exists']

        # Check that the instance type is correct.
        self.assertIsInstance(instance_running_cmd, WaiterStateCommand)
        self.assertIsInstance(bucket_exists_cmd, WaiterStateCommand)

        # Check the descriptions are set correctly.
        self.assertEqual(
            instance_running_cmd.DESCRIPTION,
            'My waiter description. It will poll every 1 seconds until '
            'a successful state has been reached. This will exit with a '
            'return code of 255 after 10 failed checks.',
        )
        self.assertEqual(
            bucket_exists_cmd.DESCRIPTION,
            'My waiter description. It will poll every 1 seconds until '
            'a successful state has been reached. This will exit with a '
            'return code of 255 after 10 failed checks.',
        )


class TestWaiterStateDocBuilder(unittest.TestCase):
    def setUp(self):
        self.waiter_config = mock.Mock()
        self.waiter_config.description = ''
        self.waiter_config.operation = 'MyOperation'
        self.waiter_config.delay = 5
        self.waiter_config.max_attempts = 20

        # Set up the acceptors.
        self.success_acceptor = mock.Mock()
        self.success_acceptor.state = 'success'
        self.fail_acceptor = mock.Mock()
        self.fail_acceptor.state = 'failure'
        self.error_acceptor = mock.Mock()
        self.error_acceptor.state = 'error'
        self.waiter_config.acceptors = [
            self.fail_acceptor,
            self.success_acceptor,
            self.error_acceptor,
        ]

        self.doc_builder = WaiterStateDocBuilder(self.waiter_config)

    def test_config_provided_description(self):
        # Description is provided by the config file
        self.waiter_config.description = 'My description.'
        description = self.doc_builder.build_waiter_state_description()
        self.assertEqual(
            description,
            'My description. It will poll every 5 seconds until a '
            'successful state has been reached. This will exit with a '
            'return code of 255 after 20 failed checks.',
        )

    def test_error_acceptor(self):
        self.success_acceptor.matcher = 'error'
        self.success_acceptor.expected = 'MyException'
        description = self.doc_builder.build_waiter_state_description()
        self.assertEqual(
            description,
            'Wait until MyException is thrown when polling with '
            '``my-operation``. It will poll every 5 seconds until a '
            'successful state has been reached. This will exit with a '
            'return code of 255 after 20 failed checks.',
        )

    def test_status_acceptor(self):
        self.success_acceptor.matcher = 'status'
        self.success_acceptor.expected = 200
        description = self.doc_builder.build_waiter_state_description()
        self.assertEqual(
            description,
            'Wait until 200 response is received when polling with '
            '``my-operation``. It will poll every 5 seconds until a '
            'successful state has been reached. This will exit with a '
            'return code of 255 after 20 failed checks.',
        )

    def test_path_acceptor(self):
        self.success_acceptor.matcher = 'path'
        self.success_acceptor.argument = 'MyResource.name'
        self.success_acceptor.expected = 'running'
        description = self.doc_builder.build_waiter_state_description()
        self.assertEqual(
            description,
            'Wait until JMESPath query MyResource.name returns running when '
            'polling with ``my-operation``. It will poll every 5 seconds '
            'until a successful state has been reached. This will exit with '
            'a return code of 255 after 20 failed checks.',
        )

    def test_path_all_acceptor(self):
        self.success_acceptor.matcher = 'pathAll'
        self.success_acceptor.argument = 'MyResource[].name'
        self.success_acceptor.expected = 'running'
        description = self.doc_builder.build_waiter_state_description()
        self.assertEqual(
            description,
            'Wait until JMESPath query MyResource[].name returns running for '
            'all elements when polling with ``my-operation``. It will poll '
            'every 5 seconds until a successful state has been reached. '
            'This will exit with a return code of 255 after 20 failed checks.',
        )

    def test_path_any_acceptor(self):
        self.success_acceptor.matcher = 'pathAny'
        self.success_acceptor.argument = 'MyResource[].name'
        self.success_acceptor.expected = 'running'
        description = self.doc_builder.build_waiter_state_description()
        self.assertEqual(
            description,
            'Wait until JMESPath query MyResource[].name returns running for '
            'any element when polling with ``my-operation``. It will poll '
            'every 5 seconds until a successful state has been reached. '
            'This will exit with a return code of 255 after 20 failed checks.',
        )


class TestWaiterCaller(unittest.TestCase):
    def test_invoke(self):
        waiter = mock.Mock()
        waiter_name = 'my_waiter'
        session = mock.Mock()
        session.create_client.return_value.get_waiter.return_value = waiter

        parameters = {'Foo': 'bar', 'Baz': 'biz'}
        parsed_globals = mock.Mock()
        parsed_globals.region = 'us-east-1'
        parsed_globals.endpoint_url = 'myurl'
        parsed_globals.verify_ssl = True

        waiter_caller = WaiterCaller(session, waiter_name)
        waiter_caller.invoke(
            'myservice', 'MyWaiter', parameters, parsed_globals
        )

        # Make sure the client was created properly.
        session.create_client.assert_called_with(
            'myservice',
            region_name=parsed_globals.region,
            endpoint_url=parsed_globals.endpoint_url,
            verify=parsed_globals.verify_ssl,
        )

        # Make sure we got the correct waiter.
        session.create_client.return_value.get_waiter.assert_called_with(
            waiter_name
        )

        # Ensure the wait command was called properly.
        waiter.wait.assert_called_with(Foo='bar', Baz='biz')



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import argparse

from awscli.customizations.emr.command import Command
from awscli.testutils import mock
from tests.unit.customizations.emr import (
    EMRBaseAWSCommandParamsTest as BaseAWSCommandParamsTest,
)


class FakeCommand(Command):
    def _run_main_command(self, parsed_args, parsed_globals):
        return 0


class TestCommand(BaseAWSCommandParamsTest):
    def test_region(self):
        def mock_region_side_effect(*args, **kwargs):
            if args[0] == 'region':
                return 'eu-central-1'
            else:
                return None

        mock_session = mock.Mock()
        mock_session.get_config_variable.side_effect = mock_region_side_effect
        mock_session.get_scoped_config.return_value = {}
        parsed_globals = argparse.Namespace()
        parsed_globals.region = None
        mocked_parsed_args = mock.Mock()

        cmd = FakeCommand(mock_session)
        cmd._run_main(mocked_parsed_args, parsed_globals)

        self.assertEqual(cmd.region, 'eu-central-1')



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest, FileCreator


class TestImortTerminology(BaseAWSCommandParamsTest):
    prefix = 'translate import-terminology'

    def setUp(self):
        super(TestImortTerminology, self).setUp()
        self.files = FileCreator()
        self.temp_file = self.files.create_file('foo', 'mycontents')
        with open(self.temp_file, 'rb') as f:
            self.temp_file_contents = f.read()

    def tearDown(self):
        super(TestImortTerminology, self).tearDown()
        self.files.remove_all()

    def test_import_terminology_with_file_and_csv(self):
        cmdline = self.prefix
        cmdline += ' --name myterminology --merge-strategy OVERWRITE'
        cmdline += ' --terminology-data Format=CSV'
        cmdline += ' --data-file fileb://%s' % self.temp_file
        result = {
            'Name': 'myterminology',
            'MergeStrategy': 'OVERWRITE',
            'TerminologyData': {
                'File': self.temp_file_contents,
                'Format': 'CSV',
            },
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_import_terminology_with_file_and_tmx(self):
        cmdline = self.prefix
        cmdline += ' --name myterminology --merge-strategy OVERWRITE'
        cmdline += ' --terminology-data Format=TMX'
        cmdline += ' --data-file fileb://%s' % self.temp_file
        result = {
            'Name': 'myterminology',
            'MergeStrategy': 'OVERWRITE',
            'TerminologyData': {
                'File': self.temp_file_contents,
                'Format': 'TMX',
            },
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_import_using_original_file_param(self):
        cmdline = self.prefix
        cmdline += ' --name myterminology --merge-strategy OVERWRITE'
        cmdline += ' --terminology-data File=fileb://wrong.csv,Format=TMX'
        cmdline += ' --data-file fileb://right.csv'
        stdout, stderr, rc = self.run_cmd(cmdline, expected_rc=252)
        self.assertIn(
            "File cannot be provided as part of the '--terminology-data' "
            "argument. Please use the '--data-file' option instead to specify "
            "a file.",
            stderr,
        )

    def test_import_terminology_with_no_file(self):
        cmdline = self.prefix
        cmdline += ' --name myterminology --merge-strategy OVERWRITE'
        cmdline += ' --terminology-data Format=TMX'
        stdout, stderr, rc = self.run_cmd(cmdline, expected_rc=252)
        self.assertIn('--data-file', stderr)

    def test_import_terminology_with_no_format(self):
        cmdline = self.prefix
        cmdline += ' --name myterminology --merge-strategy OVERWRITE'
        cmdline += ' --data-file fileb://%s' % self.temp_file
        stdout, stderr, rc = self.run_cmd(cmdline, expected_rc=252)
        self.assertIn(
            'Missing required parameter in TerminologyData: "Format"', stderr
        )



# Copyright 2023 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from tests import BaseSessionTest, ClientHTTPStubber


class BaseSQSOperationTest(BaseSessionTest):
    def setUp(self):
        super().setUp()
        self.region = "us-west-2"
        self.client = self.session.create_client("sqs", self.region)
        self.http_stubber = ClientHTTPStubber(self.client)


class SQSQueryCompatibleTest(BaseSQSOperationTest):
    def test_query_compatibility_mode_header_sent(self):
        with self.http_stubber as stub:
            stub.add_response()
            self.client.delete_queue(QueueUrl="not-a-real-queue-botocore")
            request = self.http_stubber.requests[0]
            assert 'x-amzn-query-mode' in request.headers
            assert request.headers['x-amzn-query-mode'] == b'true'



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import errno
import json

from botocore.exceptions import ProfileNotFound

from awscli.clidriver import AWSCLIEntryPoint
from awscli.testutils import (
    BaseAWSCommandParamsTest,
    BaseAWSHelpOutputTest,
    create_clidriver,
    mock,
    temporary_file,
)


class TestSessionManager(BaseAWSCommandParamsTest):
    @mock.patch('awscli.customizations.sessionmanager.check_call')
    @mock.patch("awscli.customizations.sessionmanager.check_output")
    def test_start_session_success(self, mock_check_output, mock_check_call):
        cmdline = 'ssm start-session --target instance-id'
        mock_check_call.return_value = 0
        mock_check_output.return_value = "1.2.0.0\n"
        expected_response = {
            "SessionId": "session-id",
            "TokenValue": "token-value",
            "StreamUrl": "stream-url",
        }
        self.parsed_responses = [expected_response]
        start_session_params = {"Target": "instance-id"}

        self.run_cmd(cmdline, expected_rc=0)

        mock_check_call.assert_called_once_with(
            [
                "session-manager-plugin",
                json.dumps(expected_response),
                mock.ANY,
                "StartSession",
                "",
                json.dumps(start_session_params),
                mock.ANY,
            ],
            env=self.environ,
        )

    @mock.patch("awscli.customizations.sessionmanager.check_call")
    @mock.patch("awscli.customizations.sessionmanager.check_output")
    def test_start_session_with_new_version_plugin_success(
        self, mock_check_output, mock_check_call
    ):
        cmdline = "ssm start-session --target instance-id"
        mock_check_call.return_value = 0
        mock_check_output.return_value = "1.2.500.0\n"
        expected_response = {
            "SessionId": "session-id",
            "TokenValue": "token-value",
            "StreamUrl": "stream-url",
        }
        self.parsed_responses = [expected_response]

        ssm_env_name = "AWS_SSM_START_SESSION_RESPONSE"
        start_session_params = {"Target": "instance-id"}
        expected_env = self.environ.copy()
        expected_env.update({ssm_env_name: json.dumps(expected_response)})

        self.run_cmd(cmdline, expected_rc=0)
        self.assertEqual(self.operations_called[0][0].name, 'StartSession')
        self.assertEqual(
            self.operations_called[0][1], {'Target': 'instance-id'}
        )

        mock_check_call.assert_called_once_with(
            [
                "session-manager-plugin",
                ssm_env_name,
                mock.ANY,
                "StartSession",
                "",
                json.dumps(start_session_params),
                mock.ANY,
            ],
            env=expected_env,
        )

    @mock.patch("awscli.customizations.sessionmanager.check_call")
    @mock.patch("awscli.customizations.sessionmanager.check_output")
    def test_profile_parameter_passed_to_sessionmanager_plugin(
        self, mock_check_output, mock_check_call
    ):
        cmdline = (
            "ssm start-session --target instance-id " "--profile user_profile"
        )
        mock_check_call.return_value = 0
        mock_check_output.return_value = "1.2.500.0\n"
        expected_response = {
            "SessionId": "session-id",
            "TokenValue": "token-value",
            "StreamUrl": "stream-url",
        }
        self.parsed_responses = [expected_response]
        ssm_env_name = "AWS_SSM_START_SESSION_RESPONSE"
        start_session_params = {
            "Target": "instance-id",
        }

        # We test this by creating 4 credentials
        # env vars, default profile (default),
        # env aws profile (local_profile)
        # and StartSession command profile (user_profile)
        # We want to make sure only profile name in command
        # be set to session manager plugin as parameter
        self.environ['AWS_ACCESS_KEY_ID'] = 'env_var_akid'
        self.environ['AWS_SECRET_ACCESS_KEY'] = 'env_var_sak'

        with temporary_file('w') as f:
            f.write(
                '[default]\n'
                'aws_access_key_id = shared_default_akid\n'
                'aws_secret_access_key = shared_default_sak\n'
                '[local_profile]\n'
                'aws_access_key_id = shared_local_akid\n'
                'aws_secret_access_key = shared_local_sak\n'
                '[user_profile]\n'
                'aws_access_key_id = shared_user_akid\n'
                'aws_secret_access_key = shared_user_sak\n'
            )
            f.flush()

            self.environ['AWS_SHARED_CREDENTIALS_FILE'] = f.name
            self.environ["AWS_PROFILE"] = "local_profile"

            expected_env = self.environ.copy()
            expected_env.update({ssm_env_name: json.dumps(expected_response)})

            self.driver = create_clidriver()
            self.entry_point = AWSCLIEntryPoint(self.driver)
            self.run_cmd(cmdline, expected_rc=0)

        self.assertEqual(self.operations_called[0][0].name, 'StartSession')
        self.assertEqual(
            self.operations_called[0][1], {'Target': 'instance-id'}
        )
        mock_check_call.assert_called_once_with(
            [
                "session-manager-plugin",
                ssm_env_name,
                mock.ANY,
                "StartSession",
                "user_profile",
                json.dumps(start_session_params),
                mock.ANY,
            ],
            env=expected_env,
        )

    @mock.patch("awscli.customizations.sessionmanager.check_call")
    @mock.patch("awscli.customizations.sessionmanager.check_output")
    def test_profile_environment_not_passed_to_sessionmanager_plugin(
        self, mock_check_output, mock_check_call
    ):
        cmdline = "ssm start-session --target instance-id"
        mock_check_call.return_value = 0
        mock_check_output.return_value = "1.2.500.0\n"
        expected_response = {
            "SessionId": "session-id",
            "TokenValue": "token-value",
            "StreamUrl": "stream-url",
        }
        self.parsed_responses = [expected_response]
        ssm_env_name = "AWS_SSM_START_SESSION_RESPONSE"
        start_session_params = {
            "Target": "instance-id",
        }

        with temporary_file('w') as f:
            f.write(
                '[default]\n'
                'aws_access_key_id = shared_default_akid\n'
                'aws_secret_access_key = shared_default_sak\n'
                '[local_profile]\n'
                'aws_access_key_id = shared_local_akid\n'
                'aws_secret_access_key = shared_local_sak\n'
            )
            f.flush()

            self.environ.pop("AWS_ACCESS_KEY_ID", None)
            self.environ.pop("AWS_SECRET_ACCESS_KEY", None)
            self.environ['AWS_SHARED_CREDENTIALS_FILE'] = f.name
            self.environ["AWS_PROFILE"] = "local_profile"

            expected_env = self.environ.copy()
            expected_env.update({ssm_env_name: json.dumps(expected_response)})

            self.driver = create_clidriver()
            self.entry_point = AWSCLIEntryPoint(self.driver)
            self.run_cmd(cmdline, expected_rc=0)

        self.assertEqual(self.operations_called[0][0].name, 'StartSession')
        self.assertEqual(
            self.operations_called[0][1], {'Target': 'instance-id'}
        )
        mock_check_call.assert_called_once_with(
            [
                "session-manager-plugin",
                ssm_env_name,
                mock.ANY,
                "StartSession",
                "",
                json.dumps(start_session_params),
                mock.ANY,
            ],
            env=expected_env,
        )

    @mock.patch("awscli.customizations.sessionmanager.check_call")
    @mock.patch("awscli.customizations.sessionmanager.check_output")
    def test_default_profile_used_and_not_passed_to_sessionmanager_plugin(
        self, mock_check_output, mock_check_call
    ):
        cmdline = "ssm start-session --target instance-id"
        mock_check_call.return_value = 0
        mock_check_output.return_value = "1.2.500.0\n"
        expected_response = {
            "SessionId": "session-id",
            "TokenValue": "token-value",
            "StreamUrl": "stream-url",
        }
        self.parsed_responses = [expected_response]
        ssm_env_name = "AWS_SSM_START_SESSION_RESPONSE"
        start_session_params = {
            "Target": "instance-id",
        }

        with temporary_file('w') as f:
            f.write(
                '[default]\n'
                'aws_access_key_id = shared_default_akid\n'
                'aws_secret_access_key = shared_default_sak\n'
            )
            f.flush()

            self.environ.pop("AWS_ACCESS_KEY_ID", None)
            self.environ.pop("AWS_SECRET_ACCESS_KEY", None)
            self.environ.pop("AWS_SHARED_CREDENTIALS_FILE", None)
            self.environ.pop("AWS_PROFILE", None)

            expected_env = self.environ.copy()
            expected_env.update({ssm_env_name: json.dumps(expected_response)})

            self.driver = create_clidriver()
            self.entry_point = AWSCLIEntryPoint(self.driver)
            self.run_cmd(cmdline, expected_rc=0)

        self.assertEqual(self.operations_called[0][0].name, 'StartSession')
        self.assertEqual(
            self.operations_called[0][1], {'Target': 'instance-id'}
        )
        mock_check_call.assert_called_once_with(
            [
                "session-manager-plugin",
                ssm_env_name,
                mock.ANY,
                "StartSession",
                "",
                json.dumps(start_session_params),
                mock.ANY,
            ],
            env=expected_env,
        )

    def test_start_session_with_user_profile_not_exist(self):
        cmdline = (
            "ssm start-session --target instance-id " "--profile user_profile"
        )
        with temporary_file('w') as f:
            f.write(
                '[default]\n'
                'aws_access_key_id = shared_default_akid\n'
                'aws_secret_access_key = shared_default_sak\n'
            )
            f.flush()

            self.environ.pop("AWS_ACCESS_KEY_ID", None)
            self.environ.pop("AWS_SECRET_ACCESS_KEY", None)
            self.environ.pop("AWS_PROFILE", None)
            self.environ['AWS_SHARED_CREDENTIALS_FILE'] = f.name

            try:
                self.driver = create_clidriver()
                self.entry_point = AWSCLIEntryPoint(self.driver)
                self.run_cmd(cmdline, expected_rc=255)
            except ProfileNotFound as e:
                self.assertIn(
                    'The config profile (user_profile) could not be found',
                    str(e),
                )

    @mock.patch('awscli.customizations.sessionmanager.check_call')
    @mock.patch("awscli.customizations.sessionmanager.check_output")
    def test_start_session_fails(self, mock_check_output, mock_check_call):
        cmdline = "ssm start-session --target instance-id"
        mock_check_output.return_value = "1.2.500.0\n"
        mock_check_call.side_effect = OSError(errno.ENOENT, "some error")
        self.parsed_responses = [
            {
                "SessionId": "session-id",
                "TokenValue": "token-value",
                "StreamUrl": "stream-url",
            }
        ]
        self.run_cmd(cmdline, expected_rc=255)
        self.assertEqual(self.operations_called[0][0].name, "StartSession")
        self.assertEqual(
            self.operations_called[0][1], {"Target": "instance-id"}
        )
        self.assertEqual(self.operations_called[1][0].name, "TerminateSession")
        self.assertEqual(
            self.operations_called[1][1], {"SessionId": "session-id"}
        )

    @mock.patch("awscli.customizations.sessionmanager.check_call")
    @mock.patch("awscli.customizations.sessionmanager.check_output")
    def test_start_session_when_get_plugin_version_fails(
        self, mock_check_output, mock_check_call
    ):
        cmdline = 'ssm start-session --target instance-id'
        mock_check_output.side_effect = OSError(errno.ENOENT, 'some error')
        self.parsed_responses = [
            {
                "SessionId": "session-id",
                "TokenValue": "token-value",
                "StreamUrl": "stream-url",
            }
        ]
        self.run_cmd(cmdline, expected_rc=255)
        self.assertEqual(self.operations_called[0][0].name, 'StartSession')
        self.assertEqual(
            self.operations_called[0][1], {'Target': 'instance-id'}
        )
        self.assertEqual(self.operations_called[1][0].name, 'TerminateSession')
        self.assertEqual(
            self.operations_called[1][1], {'SessionId': 'session-id'}
        )


class TestHelpOutput(BaseAWSHelpOutputTest):
    def test_start_session_output(self):
        self.driver.main(['ssm', 'start-session', 'help'])
        self.assert_contains('Output\n======\n\nNone')



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import sys
from argparse import Namespace
from socket import timeout

from awscli.customizations.codedeploy.systems import (
    RHEL,
    System,
    Ubuntu,
    Windows,
)
from awscli.customizations.codedeploy.uninstall import Uninstall
from awscli.customizations.exceptions import ConfigurationError
from awscli.testutils import mock, unittest


class TestUninstall(unittest.TestCase):
    def setUp(self):
        self.region = 'us-east-1'

        self.system_patcher = mock.patch('platform.system')
        self.system = self.system_patcher.start()
        self.system.return_value = 'Linux'

        self.linux_distribution_patcher = mock.patch(
            'awscli.compat.linux_distribution'
        )
        self.linux_distribution = self.linux_distribution_patcher.start()
        self.linux_distribution.return_value = ('Ubuntu', '', '')

        self.urlopen_patcher = mock.patch(
            'awscli.customizations.codedeploy.utils.urlopen'
        )
        self.urlopen = self.urlopen_patcher.start()
        self.urlopen.side_effect = timeout('Not EC2 instance')

        self.geteuid_patcher = mock.patch('os.geteuid', create=True)
        self.geteuid = self.geteuid_patcher.start()
        self.geteuid.return_value = 0

        self.remove_patcher = mock.patch('os.remove')
        self.remove = self.remove_patcher.start()

        self.args = Namespace()
        self.globals = Namespace()
        self.globals.region = self.region
        self.session = mock.MagicMock()
        self.uninstall = Uninstall(self.session)

    def tearDown(self):
        self.system_patcher.stop()
        self.linux_distribution_patcher.stop()
        self.urlopen_patcher.stop()
        self.geteuid_patcher.stop()
        self.remove_patcher.stop()

    def test_uninstall_throws_on_invalid_region(self):
        self.globals.region = None
        self.session.get_config_variable.return_value = None
        error_msg = 'Region not specified.'
        with self.assertRaisesRegex(ConfigurationError, error_msg):
            self.uninstall._run_main(self.args, self.globals)

    def test_uninstall_throws_on_unsupported_system(self):
        self.system.return_value = 'Unsupported'
        with self.assertRaisesRegex(
            RuntimeError, System.UNSUPPORTED_SYSTEM_MSG
        ):
            self.uninstall._run_main(self.args, self.globals)

    def test_uninstall_throws_on_ec2_instance(self):
        self.urlopen.side_effect = None
        with self.assertRaisesRegex(
            RuntimeError, 'Amazon EC2 instances are not supported.'
        ):
            self.uninstall._run_main(self.args, self.globals)
        self.assertIn('system', self.args)
        self.assertTrue(isinstance(self.args.system, Ubuntu))

    def test_uninstall_throws_on_non_administrator(self):
        self.geteuid.return_value = 1
        with self.assertRaisesRegex(
            RuntimeError, 'You must run this command as sudo.'
        ):
            self.uninstall._run_main(self.args, self.globals)

    @mock.patch.object(Ubuntu, 'uninstall')
    def test_uninstall_for_ubuntu(self, uninstall):
        self.system.return_value = 'Linux'
        self.linux_distribution.return_value = ('Ubuntu', '', '')
        self.uninstall._run_main(self.args, self.globals)
        uninstall.assert_called_with(self.args)
        self.remove.assert_called_with(
            '/etc/codedeploy-agent/conf/codedeploy.onpremises.yml'
        )

    @mock.patch.object(RHEL, 'uninstall')
    def test_uninstall_for_RHEL(self, uninstall):
        self.system.return_value = 'Linux'
        self.linux_distribution.return_value = (
            'Red Hat Enterprise Linux Server',
            '',
            '',
        )
        self.uninstall._run_main(self.args, self.globals)
        uninstall.assert_called_with(self.args)
        self.remove.assert_called_with(
            '/etc/codedeploy-agent/conf/codedeploy.onpremises.yml'
        )

    @mock.patch.object(Windows, 'uninstall')
    @mock.patch.object(Windows, 'validate_administrator')
    def test_uninstall_for_windows(self, validate_administrator, uninstall):
        self.system.return_value = 'Windows'
        self.uninstall._run_main(self.args, self.globals)
        validate_administrator.assert_called_with()
        uninstall.assert_called_with(self.args)
        self.remove.assert_called_with(
            r'C:\ProgramData\Amazon\CodeDeploy\conf.onpremises.yml'
        )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import copy
import os
import re
import sys

from awscli.testutils import BaseAWSCommandParamsTest

if sys.version_info[:2] == (2, 6):
    from StringIO import StringIO


# file is gone in python3, so instead IOBase must be used.
# Given this test module is the only place that cares about
# this type check, we do the check directly in this test module.
try:
    file_type = file
except NameError:
    import io

    file_type = io.IOBase


class TestGetObject(BaseAWSCommandParamsTest):
    prefix = 's3 cp '

    def setUp(self):
        super(TestGetObject, self).setUp()
        self.file_path = os.path.join(
            os.path.dirname(__file__), 'test_copy_params_data'
        )
        self.parsed_response = {
            'ETag': '"120ea8a25e5d487bf68b5f7096440019"',
        }

    def assert_params(self, cmdline, result):
        foo = self.assert_params_for_cmd(
            cmdline, result, expected_rc=0, ignore_params=['Body']
        )

    def test_simple(self):
        cmdline = self.prefix
        cmdline += self.file_path
        cmdline += ' s3://mybucket/mykey'
        result = {
            'Bucket': 'mybucket',
            'Key': 'mykey',
            'ChecksumAlgorithm': 'CRC64NVME',
        }
        self.assert_params(cmdline, result)

    def test_sse(self):
        cmdline = self.prefix
        cmdline += self.file_path
        cmdline += ' s3://mybucket/mykey'
        cmdline += ' --sse'
        result = {
            'Bucket': 'mybucket',
            'Key': 'mykey',
            'ServerSideEncryption': 'AES256',
            'ChecksumAlgorithm': 'CRC64NVME',
        }
        self.assert_params(cmdline, result)

    def test_storage_class(self):
        cmdline = self.prefix
        cmdline += self.file_path
        cmdline += ' s3://mybucket/mykey'
        cmdline += ' --storage-class REDUCED_REDUNDANCY'
        result = {
            'Bucket': 'mybucket',
            'Key': 'mykey',
            'ChecksumAlgorithm': 'CRC64NVME',
            'StorageClass': 'REDUCED_REDUNDANCY',
        }
        self.assert_params(cmdline, result)

    def test_standard_ia_storage_class(self):
        cmdline = self.prefix
        cmdline += self.file_path
        cmdline += ' s3://mybucket/mykey'
        cmdline += ' --storage-class STANDARD_IA'
        result = {
            'Bucket': 'mybucket',
            'Key': 'mykey',
            'StorageClass': 'STANDARD_IA',
            'ChecksumAlgorithm': 'CRC64NVME',
        }
        self.assert_params(cmdline, result)

    def test_glacier_ir_storage_class(self):
        cmdline = self.prefix
        cmdline += self.file_path
        cmdline += ' s3://mybucket/mykey'
        cmdline += ' --storage-class GLACIER_IR'
        result = {
            'Bucket': 'mybucket',
            'Key': 'mykey',
            'ChecksumAlgorithm': 'CRC64NVME',
            'StorageClass': 'GLACIER_IR',
        }
        self.assert_params(cmdline, result)

    def test_website_redirect(self):
        cmdline = self.prefix
        cmdline += self.file_path
        cmdline += ' s3://mybucket/mykey'
        cmdline += ' --website-redirect /foobar'
        result = {
            'Bucket': 'mybucket',
            'Key': 'mykey',
            'ChecksumAlgorithm': 'CRC64NVME',
            'WebsiteRedirectLocation': '/foobar',
        }
        self.assert_params(cmdline, result)

    def test_acl(self):
        cmdline = self.prefix
        cmdline += self.file_path
        cmdline += ' s3://mybucket/mykey'
        cmdline += ' --acl public-read'
        result = {
            'Bucket': 'mybucket',
            'Key': 'mykey',
            'ChecksumAlgorithm': 'CRC64NVME',
            'ACL': 'public-read',
        }
        self.assert_params(cmdline, result)

    def test_content_params(self):
        cmdline = self.prefix
        cmdline += self.file_path
        cmdline += ' s3://mybucket/mykey'
        cmdline += ' --content-encoding x-gzip'
        cmdline += ' --content-language piglatin'
        cmdline += ' --cache-control max-age=3600,must-revalidate'
        cmdline += ' --content-disposition attachment;filename="fname.ext"'
        result = {
            'Bucket': 'mybucket',
            'Key': 'mykey',
            'ChecksumAlgorithm': 'CRC64NVME',
            'ContentEncoding': 'x-gzip',
            'ContentLanguage': 'piglatin',
            'ContentDisposition': 'attachment;filename="fname.ext"',
            'CacheControl': 'max-age=3600,must-revalidate',
        }
        self.assert_params(cmdline, result)

    def test_grants(self):
        cmdline = self.prefix
        cmdline += self.file_path
        cmdline += ' s3://mybucket/mykey'
        cmdline += ' --grants read=bob'
        cmdline += ' full=alice'
        result = {
            'Bucket': 'mybucket',
            'GrantFullControl': 'alice',
            'GrantRead': 'bob',
            'Key': 'mykey',
            'ChecksumAlgorithm': 'CRC64NVME',
        }
        self.assert_params(cmdline, result)

    def test_grants_bad(self):
        cmdline = self.prefix
        cmdline += self.file_path
        cmdline += ' s3://mybucket/mykey'
        cmdline += ' --grants read:bob'
        self.assert_params_for_cmd(
            cmdline, expected_rc=1, ignore_params=['Payload']
        )

    def test_content_type(self):
        cmdline = self.prefix
        cmdline += self.file_path
        cmdline += ' s3://mybucket/mykey'
        cmdline += ' --content-type text/xml'
        result = {
            'Bucket': 'mybucket',
            'ContentType': 'text/xml',
            'Key': 'mykey',
            'ChecksumAlgorithm': 'CRC64NVME',
        }
        self.assert_params(cmdline, result)


if __name__ == "__main__":
    unittest.main()



# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestDataPipelineQueryObjects(BaseAWSCommandParamsTest):
    maxDiff = None

    prefix = 'datapipeline list-runs '

    def _generate_pipeline_objects(self, object_ids):
        objects = []
        for object_id in object_ids:
            objects.append(
                {
                    'id': object_id,
                    'name': object_id,
                    'fields': [
                        {'key': '@componentParent', 'stringValue': object_id}
                    ],
                }
            )
        return objects

    def test_list_more_than_one_hundred_runs(self):
        start_date = '2017-10-22T00:37:21'
        end_date = '2017-10-26T00:37:21'
        pipeline_id = 'pipeline-id'
        args = '--pipeline-id %s --start-interval %s,%s' % (
            pipeline_id,
            start_date,
            end_date,
        )
        command = self.prefix + args
        object_ids = ['object-id-%s' % i for i in range(150)]
        objects = self._generate_pipeline_objects(object_ids)

        self.parsed_responses = [
            {
                'ids': object_ids[:100],
                'hasMoreResults': True,
                'marker': 'marker',
            },
            {'ids': object_ids[100:], 'hasMoreResults': False},
            {'pipelineObjects': objects[:100]},
            {'pipelineObjects': objects[100:]},
        ]

        self.run_cmd(command, expected_rc=0)

        query = {
            'selectors': [
                {
                    'fieldName': '@actualStartTime',
                    'operator': {
                        'type': 'BETWEEN',
                        'values': [start_date, end_date],
                    },
                }
            ]
        }

        expected_operations_called = [
            (
                'QueryObjects',
                {
                    'pipelineId': pipeline_id,
                    'query': query,
                    'sphere': 'INSTANCE',
                },
            ),
            (
                'QueryObjects',
                {
                    'pipelineId': pipeline_id,
                    'marker': 'marker',
                    'query': query,
                    'sphere': 'INSTANCE',
                },
            ),
            (
                'DescribeObjects',
                {'objectIds': object_ids[:100], 'pipelineId': pipeline_id},
            ),
            (
                'DescribeObjects',
                {'objectIds': object_ids[100:], 'pipelineId': pipeline_id},
            ),
        ]
        operations_called = [
            (op.name, params) for op, params in self.operations_called
        ]
        self.assertEqual(expected_operations_called, operations_called)



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestPutIntegration(BaseAWSCommandParamsTest):
    prefix = 'apigateway put-integration '

    def test_put_integration(self):
        cmdline = self.prefix
        cmdline += '--rest-api-id api-id '
        cmdline += '--resource-id resource-id '
        cmdline += '--http-method GET '
        cmdline += '--type HTTP '
        cmdline += '--integration-http-method GET '
        cmdline += '--uri https://api.endpoint.com'
        result = {
            'restApiId': 'api-id',
            'resourceId': 'resource-id',
            'httpMethod': 'GET',
            'type': 'HTTP',
            'integrationHttpMethod': 'GET',
            'uri': 'https://api.endpoint.com',
        }
        self.assert_params_for_cmd(cmdline, result)



# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import sys

from botocore import hooks

from awscli import plugin
from awscli.testutils import mock, unittest


class FakeModule:
    def __init__(self):
        self.called = False
        self.context = None
        self.events_seen = []

    def awscli_initialize(self, context):
        self.called = True
        self.context = context
        self.context.register(
            'before_operation',
            (lambda **kwargs: self.events_seen.append(kwargs)),
        )


class TestPlugins(unittest.TestCase):
    def setUp(self):
        self.fake_module = FakeModule()
        sys.modules['__fake_plugin__'] = self.fake_module
        self.plugin_mapping = {
            'cli_legacy_plugin_path': 'fake-path',
            'fake_plugin': '__fake_plugin__',
        }

    def tearDown(self):
        del sys.modules['__fake_plugin__']

    def test_plugin_register(self):
        emitter = plugin.load_plugins(self.plugin_mapping)
        self.assertTrue(self.fake_module.called)
        self.assertTrue(isinstance(emitter, hooks.HierarchicalEmitter))
        self.assertTrue(
            isinstance(self.fake_module.context, hooks.HierarchicalEmitter)
        )

    def test_event_hooks_can_be_passed_in(self):
        hooks = plugin.HierarchicalEmitter()
        emitter = plugin.load_plugins(self.plugin_mapping, event_hooks=hooks)
        emitter.emit('before_operation')
        self.assertEqual(len(self.fake_module.events_seen), 1)

    def test_plugins_not_registered_if_cli_legacy_plugin_path_not_set(self):
        del self.plugin_mapping['cli_legacy_plugin_path']
        plugin.load_plugins(self.plugin_mapping)
        self.assertFalse(self.fake_module.called)


class TestPluginCanBePackage(unittest.TestCase):
    def setUp(self):
        self.fake_module = FakeModule()
        self.fake_package = mock.Mock()
        sys.modules['__fake_plugin__'] = self.fake_package
        sys.modules['__fake_plugin__.__fake__'] = self.fake_package
        sys.modules['__fake_plugin__.__fake__.bar'] = self.fake_module
        self.plugin_mapping = {
            'cli_legacy_plugin_path': 'fake-path',
            'fake_plugin': '__fake_plugin__.__fake__.bar',
        }

    def tearDown(self):
        del sys.modules['__fake_plugin__.__fake__']

    def test_plugin_register(self):
        plugin.load_plugins(
            {
                'cli_legacy_plugin_path': 'fake-path',
                'fake_plugin': '__fake_plugin__.__fake__.bar',
            }
        )
        self.assertTrue(self.fake_module.called)


if __name__ == '__main__':
    unittest.main()



# Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from argparse import Namespace

from awscli.customizations.ec2.paginate import EC2PageSizeInjector
from awscli.testutils import mock, unittest


class TestEC2PageSizeInjector(unittest.TestCase):
    def test_register(self):
        target_operations = {'foo': [], 'bar': []}
        injector = EC2PageSizeInjector()
        injector.TARGET_OPERATIONS = target_operations
        event_emitter = mock.Mock()
        injector.register(event_emitter)

        call_args = event_emitter.register_last.call_args_list
        events_registered = sorted([c[0][0] for c in call_args])
        expected_events = sorted(
            ['calling-command.ec2.bar', 'calling-command.ec2.foo']
        )
        self.assertEqual(events_registered, expected_events)

    def test_inject(self):
        target_operations = {'foo': []}
        injector = EC2PageSizeInjector()
        injector.DEFAULT_PAGE_SIZE = 5
        injector.TARGET_OPERATIONS = target_operations
        parsed_globals = Namespace(paginate=True)
        call_parameters = {}
        event_name = 'operation-args-parsed.ec2.foo'
        injector.inject(
            event_name=event_name,
            parsed_globals=parsed_globals,
            call_parameters=call_parameters,
        )
        page_size = call_parameters.get('PaginationConfig', {}).get('PageSize')
        self.assertEqual(page_size, 5)

    def test_no_paginate(self):
        target_operations = {'foo': []}
        injector = EC2PageSizeInjector()
        injector.TARGET_OPERATIONS = target_operations
        parsed_globals = Namespace(paginate=False)
        call_parameters = {}
        event_name = 'operation-args-parsed.ec2.foo'
        injector.inject(
            event_name=event_name,
            parsed_globals=parsed_globals,
            call_parameters=call_parameters,
        )
        page_size = call_parameters.get('PaginationConfig', {}).get('PageSize')
        self.assertIsNone(page_size)

    def test_global_whitelist(self):
        target_operations = {'foo': []}
        injector = EC2PageSizeInjector()
        injector.UNIVERSAL_WHITELIST = ['bar']
        injector.TARGET_OPERATIONS = target_operations
        parsed_globals = Namespace(paginate=True)
        call_parameters = {'baz': True}
        event_name = 'operation-args-parsed.ec2.foo'
        injector.inject(
            event_name=event_name,
            parsed_globals=parsed_globals,
            call_parameters=call_parameters,
        )
        page_size = call_parameters.get('PaginationConfig', {}).get('PageSize')
        self.assertIsNone(page_size)

    def test_operation_whitelist(self):
        target_operations = {'foo': ['bar']}
        injector = EC2PageSizeInjector()
        injector.UNIVERSAL_WHITELIST = []
        injector.TARGET_OPERATIONS = target_operations
        parsed_globals = Namespace(paginate=True)
        call_parameters = {'baz': True}
        event_name = 'operation-args-parsed.ec2.foo'
        injector.inject(
            event_name=event_name,
            parsed_globals=parsed_globals,
            call_parameters=call_parameters,
        )
        page_size = call_parameters.get('PaginationConfig', {}).get('PageSize')
        self.assertIsNone(page_size)

    def test_non_target_operation(self):
        target_operations = {'foo': []}
        injector = EC2PageSizeInjector()
        injector.TARGET_OPERATIONS = target_operations
        parsed_globals = Namespace(paginate=True)
        call_parameters = {}
        event_name = 'operation-args-parsed.ec2.baz'
        injector.inject(
            event_name=event_name,
            parsed_globals=parsed_globals,
            call_parameters=call_parameters,
        )
        page_size = call_parameters.get('PaginationConfig', {}).get('PageSize')
        self.assertIsNone(page_size)



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import subprocess
from argparse import Namespace

from awscli.customizations.codedeploy.systems import RHEL, Ubuntu, Windows
from awscli.testutils import mock, unittest


class TestWindows(unittest.TestCase):
    def setUp(self):
        self.popen_patcher = mock.patch('subprocess.Popen')
        self.popen = self.popen_patcher.start()

        self.check_call_patcher = mock.patch('subprocess.check_call')
        self.check_call = self.check_call_patcher.start()

        self.open_patcher = mock.patch(
            'awscli.customizations.codedeploy.systems.open',
            mock.mock_open(),
            create=True,
        )
        self.open = self.open_patcher.start()

        self.config_dir = r'C:\ProgramData\Amazon\CodeDeploy'
        self.config_file = 'conf.onpremises.yml'
        self.config_path = rf'{self.config_dir}\{self.config_file}'
        self.installer = 'codedeploy-agent.msi'
        self.bucket = 'bucket'
        self.key = 'key'
        self.region = 'us-east-1'

        self.body = 'install-script'
        self.reader = mock.MagicMock()
        self.reader.read.return_value = self.body
        self.s3 = mock.MagicMock()
        self.s3.get_object.return_value = {'Body': self.reader}

        self.session = mock.MagicMock()
        self.session.create_client.return_value = self.s3

        self.params = Namespace()
        self.params.session = self.session
        self.params.region = self.region
        self.params.bucket = self.bucket
        self.params.key = self.key

        self.windows = Windows(self.params)

    def tearDown(self):
        self.popen_patcher.stop()
        self.check_call_patcher.stop()
        self.open_patcher.stop()

    def test_config_dir(self):
        self.assertEqual(self.config_dir, self.windows.CONFIG_DIR)

    def test_config_file(self):
        self.assertEqual(self.config_file, self.windows.CONFIG_FILE)

    def test_config_path(self):
        self.assertEqual(self.config_path, self.windows.CONFIG_PATH)

    def test_installer(self):
        self.assertEqual(self.installer, self.windows.INSTALLER)

    def test_install(self):
        process = mock.MagicMock()
        process.communicate.side_effect = [('', ''), ('Running', '')]
        process.returncode = 0
        self.popen.return_value = process
        self.windows.install(self.params)
        self.popen.assert_has_calls(
            [
                mock.call(
                    [
                        'powershell.exe',
                        '-Command',
                        'Stop-Service',
                        '-Name',
                        'codedeployagent',
                    ],
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                ),
                mock.call().communicate(),
                mock.call(
                    [
                        'powershell.exe',
                        '-Command',
                        'Get-Service',
                        '-Name',
                        'codedeployagent',
                    ],
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                ),
                mock.call().communicate(),
            ]
        )
        self.check_call.assert_has_calls(
            [
                mock.call(
                    [
                        rf'.\{self.installer}',
                        '/quiet',
                        '/l',
                        r'.\codedeploy-agent-install-log.txt',
                    ],
                    shell=True,
                ),
                mock.call(
                    [
                        'powershell.exe',
                        '-Command',
                        'Restart-Service',
                        '-Name',
                        'codedeployagent',
                    ]
                ),
            ]
        )
        self.open.assert_called_with(self.installer, 'wb')
        self.open().write.assert_called_with(self.body)

    def test_uninstall(self):
        process = mock.MagicMock()
        process.communicate.side_effect = [('', ''), ('', '')]
        process.returncode = 0
        self.popen.return_value = process
        self.windows.uninstall(self.params)
        self.popen.assert_has_calls(
            [
                mock.call(
                    [
                        'powershell.exe',
                        '-Command',
                        'Stop-Service',
                        '-Name',
                        'codedeployagent',
                    ],
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                ),
                mock.call().communicate(),
                mock.call(
                    [
                        'wmic',
                        'product',
                        'where',
                        'name="CodeDeploy Host Agent"',
                        'call',
                        'uninstall',
                        '/nointeractive',
                    ],
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                ),
                mock.call().communicate(),
            ]
        )


class TestLinux(unittest.TestCase):
    def setUp(self):
        self.popen_patcher = mock.patch('subprocess.Popen')
        self.popen = self.popen_patcher.start()

        self.check_call_patcher = mock.patch('subprocess.check_call')
        self.check_call = self.check_call_patcher.start()

        self.open_patcher = mock.patch(
            'awscli.customizations.codedeploy.systems.open',
            mock.mock_open(),
            create=True,
        )
        self.open = self.open_patcher.start()

        self.environ_patcher = mock.patch('os.environ')
        self.environ = self.environ_patcher.start()
        self.environ.copy.return_value = dict()

        self.config_dir = '/etc/codedeploy-agent/conf'
        self.config_file = 'codedeploy.onpremises.yml'
        self.config_path = f'{self.config_dir}/{self.config_file}'
        self.installer = 'install'
        self.bucket = 'bucket'
        self.key = 'key'
        self.region = 'us-east-1'

        self.access_key_id = 'ACCESSKEYID'
        self.secret_access_key = 'SECRETACCESSKEY'
        self.session_token = 'SESSION_TOKEN'
        self.credentials = mock.MagicMock()
        self.credentials.access_key = self.access_key_id
        self.credentials.secret_key = self.secret_access_key
        self.credentials.token = self.session_token

        self.environment = dict(
            {
                'AWS_REGION': self.region,
                'AWS_ACCESS_KEY_ID': self.access_key_id,
                'AWS_SECRET_ACCESS_KEY': self.secret_access_key,
                'AWS_SESSION_TOKEN': self.session_token,
            }
        )

        self.body = 'install-script'
        self.reader = mock.MagicMock()
        self.reader.read.return_value = self.body
        self.s3 = mock.MagicMock()
        self.s3.get_object.return_value = {'Body': self.reader}

        self.session = mock.MagicMock()
        self.session.create_client.return_value = self.s3
        self.session.get_credentials.return_value = self.credentials

        self.params = Namespace()
        self.params.session = self.session
        self.params.region = self.region
        self.params.bucket = self.bucket
        self.params.key = self.key

    def tearDown(self):
        self.popen_patcher.stop()
        self.check_call_patcher.stop()
        self.open_patcher.stop()
        self.environ_patcher.stop()


class TestUbuntu(TestLinux):
    def setUp(self):
        super(self.__class__, self).setUp()
        self.ubuntu = Ubuntu(self.params)

    def test_config_dir(self):
        self.assertEqual(self.config_dir, self.ubuntu.CONFIG_DIR)

    def test_config_file(self):
        self.assertEqual(self.config_file, self.ubuntu.CONFIG_FILE)

    def test_config_path(self):
        self.assertEqual(self.config_path, self.ubuntu.CONFIG_PATH)

    def test_installer(self):
        self.assertEqual(self.installer, self.ubuntu.INSTALLER)

    @mock.patch('os.geteuid', create=True)
    def test_validate_administrator_throws(self, geteuid):
        geteuid.return_value = 1
        with self.assertRaisesRegex(
            RuntimeError, 'You must run this command as sudo.'
        ):
            self.ubuntu.validate_administrator()

    def test_install(self):
        process = mock.MagicMock()
        process.communicate.return_value = ('', '')
        process.returncode = 0
        self.popen.return_value = process
        self.ubuntu.install(self.params)
        self.popen.assert_has_calls(
            [
                mock.call(
                    ['service', 'codedeploy-agent', 'stop'],
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                ),
                mock.call().communicate(),
            ]
        )
        self.check_call.assert_has_calls(
            [
                mock.call(['apt-get', '-y', 'update']),
                mock.call(['apt-get', '-y', 'install', 'ruby2.0']),
                mock.call(['chmod', '+x', f'./{self.installer}']),
                mock.call(
                    [f'./{self.installer}', 'auto'], env=self.environment
                ),
            ]
        )
        self.open.assert_called_with(self.installer, 'wb')
        self.open().write.assert_called_with(self.body)

    def test_uninstall(self):
        process = mock.MagicMock()
        process.communicate.return_value = ('', '')
        process.returncode = 0
        self.popen.return_value = process
        self.ubuntu.uninstall(self.params)
        self.popen.assert_has_calls(
            [
                mock.call(
                    ['service', 'codedeploy-agent', 'stop'],
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                ),
                mock.call().communicate(),
            ]
        )
        self.check_call.assert_has_calls(
            [mock.call(['dpkg', '-r', 'codedeploy-agent'])]
        )


class TestRHEL(TestLinux):
    def setUp(self):
        super(self.__class__, self).setUp()
        self.rhel = RHEL(self.params)

    def test_config_dir(self):
        self.assertEqual(self.config_dir, self.rhel.CONFIG_DIR)

    def test_config_file(self):
        self.assertEqual(self.config_file, self.rhel.CONFIG_FILE)

    def test_config_path(self):
        self.assertEqual(self.config_path, self.rhel.CONFIG_PATH)

    def test_installer(self):
        self.assertEqual(self.installer, self.rhel.INSTALLER)

    @mock.patch('os.geteuid', create=True)
    def test_validate_administrator_throws(self, geteuid):
        geteuid.return_value = 1
        with self.assertRaisesRegex(
            RuntimeError, 'You must run this command as sudo.'
        ):
            self.rhel.validate_administrator()

    def test_install(self):
        process = mock.MagicMock()
        process.communicate.return_value = ('', '')
        process.returncode = 0
        self.popen.return_value = process
        self.rhel.install(self.params)
        self.popen.assert_has_calls(
            [
                mock.call(
                    ['service', 'codedeploy-agent', 'stop'],
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                ),
                mock.call().communicate(),
            ]
        )
        self.check_call.assert_has_calls(
            [
                mock.call(['yum', '-y', 'install', 'ruby']),
                mock.call(['chmod', '+x', f'./{self.installer}']),
                mock.call(
                    [f'./{self.installer}', 'auto'], env=self.environment
                ),
            ]
        )
        self.open.assert_called_with(self.installer, 'wb')
        self.open().write.assert_called_with(self.body)

    def test_uninstall(self):
        process = mock.MagicMock()
        process.communicate.return_value = ('', '')
        process.returncode = 0
        self.popen.return_value = process
        self.rhel.uninstall(self.params)
        self.popen.assert_has_calls(
            [
                mock.call(
                    ['service', 'codedeploy-agent', 'stop'],
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                ),
                mock.call().communicate(),
            ]
        )
        self.check_call.assert_has_calls(
            [mock.call(['yum', '-y', 'erase', 'codedeploy-agent'])]
        )


if __name__ == "__main__":
    unittest.main()



# Copyright 2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from awscli.autocomplete.local.fetcher import CliDriverFetcher
from awscli.testutils import mock, unittest


class FakeDriver:
    def __init__(self):
        global_arg = mock.Mock()
        global_arg.documentation = 'global arg doc'
        self.arg_table = {'arg': global_arg}

        help_command = mock.Mock()
        help_command.obj = 'fake_operational_model'

        arg = mock.Mock()
        arg.documentation = 'arg doc'
        arg_table = {'instance-ids': arg}

        sub_subcommand = mock.Mock()
        sub_subcommand.arg_table = arg_table
        sub_subcommand.create_help_command.return_value = help_command

        subcommand = mock.Mock()
        subcommand.subcommand_table = {'describe-instances': sub_subcommand}
        subcommand_table = {'ec2': subcommand}
        self.subcommand_table = subcommand_table


class TestCliDriverFetcher(unittest.TestCase):
    def setUp(self) -> None:
        self.fetcher = CliDriverFetcher(FakeDriver())

    def test_get_argument_documentation(self):
        help_text = self.fetcher.get_argument_documentation(
            ['aws', 'ec2'], 'describe-instances', 'instance-ids'
        )
        self.assertEqual(help_text, 'arg doc')

    def test_get_global_arg_documentation(self):
        help_text = self.fetcher.get_global_arg_documentation('arg')
        self.assertEqual(help_text, 'global arg doc')

    def test_get_operational_model(self):
        operational_model = self.fetcher.get_operation_model(
            ['aws', 'ec2'], 'describe-instances'
        )
        self.assertEqual(operational_model, 'fake_operational_model')



# Copyright 2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import json

from botocore import model, xform_name
from botocore.compat import OrderedDict

from awscli.argprocess import (
    ParamError,
    ParamShorthandDocGen,
    ParamShorthandParser,
    ParamUnknownKeyError,
    detect_shape_structure,
    unpack_cli_arg,
)
from awscli.arguments import (
    BooleanArgument,
    CLIArgument,
    CustomArgument,
    ListArgument,
    create_argument_model_from_schema,
)
from awscli.help import OperationHelpCommand
from awscli.paramfile import LOCAL_PREFIX_MAP, URIArgumentHandler
from awscli.testutils import BaseCLIDriverTest, mock, temporary_file, unittest
from awscli.argparser import ArgTableArgParser


# These tests use real service types so that we can
# verify the real shapes of services.
class BaseArgProcessTest(BaseCLIDriverTest):
    def get_param_model(self, dotted_name):
        service_name, operation_name, param_name = dotted_name.split('.')
        service_model = self.session.get_service_model(service_name)
        operation = service_model.operation_model(operation_name)
        input_shape = operation.input_shape
        required_arguments = input_shape.required_members
        is_required = param_name in required_arguments
        member_shape = input_shape.members[param_name]
        type_name = member_shape.type_name
        cli_arg_name = xform_name(param_name, '-')
        if type_name == 'boolean':
            cls = BooleanArgument
        elif type_name == 'list':
            cls = ListArgument
        else:
            cls = CLIArgument
        return cls(cli_arg_name, member_shape, mock.Mock(), is_required)

    def create_argument(self, argument_model, argument_name=None):
        if argument_name is None:
            argument_name = 'foo'
        argument = mock.Mock()
        m = model.DenormalizedStructureBuilder().with_members(argument_model)
        argument.argument_model = m.build_model()
        argument.name = argument_name
        argument.cli_name = "--" + argument_name
        return argument


class TestURIParams(BaseArgProcessTest):
    def setUp(self):
        super(TestURIParams, self).setUp()
        self.uri_param = URIArgumentHandler(LOCAL_PREFIX_MAP.copy())

    def test_uri_param(self):
        p = self.get_param_model('ec2.DescribeInstances.Filters')
        with temporary_file('r+') as f:
            json_argument = json.dumps(
                [{"Name": "instance-id", "Values": ["i-1234"]}]
            )
            f.write(json_argument)
            f.flush()
            result = self.uri_param('event-name', p, 'file://%s' % f.name)
        self.assertEqual(result, json_argument)


class TestArgShapeDetection(BaseArgProcessTest):
    def assert_shape_type(self, spec, expected_type):
        p = self.get_param_model(spec)
        actual_structure = detect_shape_structure(p.argument_model)
        self.assertEqual(actual_structure, expected_type)

    def assert_custom_shape_type(self, schema, expected_type):
        argument_model = create_argument_model_from_schema(schema)
        argument = CustomArgument('test', argument_model=argument_model)
        actual_structure = detect_shape_structure(argument.argument_model)
        self.assertEqual(actual_structure, expected_type)

    def test_detect_scalar(self):
        self.assert_shape_type(
            'iam.AddRoleToInstanceProfile.RoleName', 'scalar'
        )

    def test_detect_list_of_strings(self):
        self.assert_shape_type('sns.AddPermission.AWSAccountId', 'list-scalar')

    def test_detect_structure_of_scalars(self):
        self.assert_shape_type(
            'elasticbeanstalk.CreateConfigurationTemplate.SourceConfiguration',
            'structure(scalars)',
        )

    def test_list_structure_scalars(self):
        self.assert_shape_type(
            'elb.RegisterInstancesWithLoadBalancer.Instances',
            'list-structure(scalar)',
        )

    def test_list_structure_scalars_2(self):
        self.assert_shape_type(
            'elb.CreateLoadBalancer.Listeners', 'list-structure(scalars)'
        )

    def test_list_structure_of_list_and_strings(self):
        self.assert_shape_type(
            'ec2.DescribeInstances.Filters',
            'list-structure(list-scalar, scalar)',
        )

    def test_map_scalar(self):
        self.assert_shape_type(
            'sqs.SetQueueAttributes.Attributes', 'map-scalar'
        )

    def test_struct_list_scalar(self):
        self.assert_custom_shape_type(
            {
                "type": "object",
                "properties": {
                    "Consistent": {
                        "type": "boolean",
                    },
                    "Args": {"type": "array", "items": {"type": "string"}},
                },
            },
            'structure(list-scalar, scalar)',
        )

    def test_recursive_shape(self):
        shapes = {
            'InputStructure': {
                'type': 'structure',
                'members': {'A': {'shape': 'RecursiveShape'}},
            },
            'RecursiveShape': {
                'type': 'structure',
                'members': {
                    'B': {'shape': 'StringType'},
                    'C': {'shape': 'RecursiveShape'},
                },
            },
            'StringType': {'type': 'string'},
        }
        shape = model.StructureShape(
            shape_name='InputStructure',
            shape_model=shapes['InputStructure'],
            shape_resolver=model.ShapeResolver(shape_map=shapes),
        )
        self.assertIn('recursive', detect_shape_structure(shape))


class TestParamShorthand(BaseArgProcessTest):
    maxDiff = None

    def setUp(self):
        super(TestParamShorthand, self).setUp()
        self._shorthand = ParamShorthandParser()

    def parse_shorthand(self, cli_argument, value, event_name=None):
        event = event_name
        if event is None:
            event = 'process-cli-arg.foo.bar'
        return self._shorthand(cli_argument, value, event)

    def test_simplify_structure_scalars(self):
        p = self.get_param_model(
            'elasticbeanstalk.CreateConfigurationTemplate.SourceConfiguration'
        )
        value = 'ApplicationName=foo,TemplateName=bar'
        json_value = '{"ApplicationName": "foo", "TemplateName": "bar"}'
        returned = self.parse_shorthand(p, value)
        json_version = unpack_cli_arg(p, json_value)
        self.assertEqual(returned, json_version)

    def test_flattens_marked_single_member_structure_list(self):
        argument = self.create_argument(
            {
                'Arg': {
                    'type': 'list',
                    'member': {
                        'type': 'structure',
                        'members': {'Bar': {'type': 'string'}},
                    },
                }
            },
            'arg',
        )
        argument.argument_model = argument.argument_model.members['Arg']
        value = ['foo', 'baz']
        uses_old_list = 'awscli.argprocess.ParamShorthand._uses_old_list_case'
        with mock.patch(uses_old_list, mock.Mock(return_value=True)):
            returned = self.parse_shorthand(argument, value)
        self.assertEqual(returned, [{"Bar": "foo"}, {"Bar": "baz"}])

    def test_does_not_flatten_unmarked_single_member_structure_list(self):
        argument = self.create_argument(
            {
                'Arg': {
                    'type': 'list',
                    'member': {
                        'type': 'structure',
                        'members': {'Bar': {'type': 'string'}},
                    },
                }
            },
            'arg',
        )
        argument.argument_model = argument.argument_model.members['Arg']
        value = ['Bar=foo', 'Bar=baz']
        uses_old_list = 'awscli.argprocess.ParamShorthand._uses_old_list_case'
        with mock.patch(uses_old_list, mock.Mock(return_value=False)):
            returned = self.parse_shorthand(argument, value)
        self.assertEqual(returned, [{"Bar": "foo"}, {"Bar": "baz"}])

    def test_parse_boolean_shorthand(self):
        bool_param = mock.Mock()
        bool_param.cli_type_name = 'boolean'
        bool_param.argument_model.type_name = 'boolean'
        bool_param.argument_model.is_document_type = False
        self.assertTrue(unpack_cli_arg(bool_param, True))
        self.assertTrue(unpack_cli_arg(bool_param, 'True'))
        self.assertTrue(unpack_cli_arg(bool_param, 'true'))

        self.assertFalse(unpack_cli_arg(bool_param, False))
        self.assertFalse(unpack_cli_arg(bool_param, 'False'))
        self.assertFalse(unpack_cli_arg(bool_param, 'false'))

    def test_simplify_map_scalar(self):
        p = self.get_param_model('sqs.SetQueueAttributes.Attributes')
        returned = self.parse_shorthand(p, 'VisibilityTimeout=15')
        json_version = unpack_cli_arg(p, '{"VisibilityTimeout": "15"}')
        self.assertEqual(returned, {'VisibilityTimeout': '15'})
        self.assertEqual(returned, json_version)

    def test_list_structure_scalars(self):
        p = self.get_param_model(
            'elb.RegisterInstancesWithLoadBalancer.Instances'
        )
        event_name = (
            'process-cli-arg.elb.register-instances-with-load-balancer'
        )
        # Because this is a list type param, we'll use nargs
        # with argparse which means the value will be presented
        # to us as a list.
        returned = self.parse_shorthand(
            p, ['instance-1', 'instance-2'], event_name
        )
        self.assertEqual(
            returned,
            [{'InstanceId': 'instance-1'}, {'InstanceId': 'instance-2'}],
        )

    def test_list_structure_list_scalar(self):
        p = self.get_param_model('ec2.DescribeInstances.Filters')
        expected = [
            {"Name": "instance-id", "Values": ["i-1", "i-2"]},
            {"Name": "architecture", "Values": ["i386"]},
        ]
        returned = self.parse_shorthand(
            p,
            [
                "Name=instance-id,Values=i-1,i-2",
                "Name=architecture,Values=i386",
            ],
        )
        self.assertEqual(returned, expected)

        # With spaces around the comma.
        returned2 = self.parse_shorthand(
            p,
            [
                "Name=instance-id, Values=i-1,i-2",
                "Name=architecture, Values=i386",
            ],
        )
        self.assertEqual(returned2, expected)

        # Strip off leading/trailing spaces.
        returned3 = self.parse_shorthand(
            p,
            [
                "Name = instance-id, Values = i-1,i-2",
                "Name = architecture, Values = i386",
            ],
        )
        self.assertEqual(returned3, expected)

    def test_parse_empty_values(self):
        # A value can be omitted and will default to an empty string.
        p = self.get_param_model('ec2.DescribeInstances.Filters')
        expected = [
            {"Name": "", "Values": ["i-1", "i-2"]},
            {"Name": "architecture", "Values": ['']},
        ]
        returned = self.parse_shorthand(
            p, ["Name=,Values=i-1,i-2", "Name=architecture,Values="]
        )
        self.assertEqual(returned, expected)

    def test_list_structure_list_scalar_2(self):
        p = self.get_param_model('emr.ModifyInstanceGroups.InstanceGroups')
        expected = [
            {"InstanceGroupId": "foo", "InstanceCount": 4},
            {"InstanceGroupId": "bar", "InstanceCount": 1},
        ]

        simplified = self.parse_shorthand(
            p,
            [
                "InstanceGroupId=foo,InstanceCount=4",
                "InstanceGroupId=bar,InstanceCount=1",
            ],
        )

        self.assertEqual(simplified, expected)

    def test_empty_value_of_list_structure(self):
        p = self.get_param_model('emr.ModifyInstanceGroups.InstanceGroups')
        expected = []
        simplified = self.parse_shorthand(p, [])
        self.assertEqual(simplified, expected)

    def test_list_structure_list_multiple_scalar(self):
        p = self.get_param_model('emr.ModifyInstanceGroups.InstanceGroups')
        returned = self.parse_shorthand(
            p,
            [
                'InstanceGroupId=foo,InstanceCount=3,'
                'EC2InstanceIdsToTerminate=i-12345,i-67890'
            ],
        )
        self.assertEqual(
            returned,
            [
                {
                    'EC2InstanceIdsToTerminate': ['i-12345', 'i-67890'],
                    'InstanceGroupId': 'foo',
                    'InstanceCount': 3,
                }
            ],
        )

    def test_list_structure_scalars_2(self):
        p = self.get_param_model('elb.CreateLoadBalancer.Listeners')
        expected = [
            {
                "Protocol": "protocol1",
                "LoadBalancerPort": 1,
                "InstanceProtocol": "instance_protocol1",
                "InstancePort": 2,
                "SSLCertificateId": "ssl_certificate_id1",
            },
            {
                "Protocol": "protocol2",
                "LoadBalancerPort": 3,
                "InstanceProtocol": "instance_protocol2",
                "InstancePort": 4,
                "SSLCertificateId": "ssl_certificate_id2",
            },
        ]
        returned = unpack_cli_arg(
            p,
            [
                '{"Protocol": "protocol1", "LoadBalancerPort": 1, '
                '"InstanceProtocol": "instance_protocol1", '
                '"InstancePort": 2, "SSLCertificateId": '
                '"ssl_certificate_id1"}',
                '{"Protocol": "protocol2", "LoadBalancerPort": 3, '
                '"InstanceProtocol": "instance_protocol2", '
                '"InstancePort": 4, "SSLCertificateId": '
                '"ssl_certificate_id2"}',
            ],
        )
        self.assertEqual(returned, expected)
        simplified = self.parse_shorthand(
            p,
            [
                'Protocol=protocol1,LoadBalancerPort=1,'
                'InstanceProtocol=instance_protocol1,'
                'InstancePort=2,SSLCertificateId=ssl_certificate_id1',
                'Protocol=protocol2,LoadBalancerPort=3,'
                'InstanceProtocol=instance_protocol2,'
                'InstancePort=4,SSLCertificateId=ssl_certificate_id2',
            ],
        )
        self.assertEqual(simplified, expected)

    def test_keyval_with_long_values(self):
        p = self.get_param_model('dynamodb.UpdateTable.ProvisionedThroughput')
        value = 'WriteCapacityUnits=10,ReadCapacityUnits=10'
        returned = self.parse_shorthand(p, value)
        self.assertEqual(
            returned, {'WriteCapacityUnits': 10, 'ReadCapacityUnits': 10}
        )

    def test_error_messages_for_structure_scalar(self):
        p = self.get_param_model(
            'elasticbeanstalk.CreateConfigurationTemplate.SourceConfiguration'
        )
        value = 'ApplicationName:foo,TemplateName=bar'
        error_msg = (
            "Error parsing parameter '--source-configuration'.*Expected"
        )
        with self.assertRaisesRegex(ParamError, error_msg):
            self.parse_shorthand(p, value)

    def test_improper_separator(self):
        # If the user uses ':' instead of '=', we should give a good
        # error message.
        p = self.get_param_model(
            'elasticbeanstalk.CreateConfigurationTemplate.SourceConfiguration'
        )
        value = 'ApplicationName:foo,TemplateName:bar'
        error_msg = (
            "Error parsing parameter '--source-configuration'.*Expected"
        )
        with self.assertRaisesRegex(ParamError, error_msg):
            self.parse_shorthand(p, value)

    def test_improper_separator_for_filters_param(self):
        p = self.get_param_model('ec2.DescribeInstances.Filters')
        error_msg = "Error parsing parameter '--filters'.*Expected"
        with self.assertRaisesRegex(ParamError, error_msg):
            self.parse_shorthand(p, ["Name:tag:Name,Values:foo"])

    def test_csv_syntax_escaped(self):
        p = self.get_param_model('cloudformation.CreateStack.Parameters')
        returned = self.parse_shorthand(
            p, [r"ParameterKey=key,ParameterValue=foo\,bar"]
        )
        expected = [{"ParameterKey": "key", "ParameterValue": "foo,bar"}]
        self.assertEqual(returned, expected)

    def test_csv_syntax_double_quoted(self):
        p = self.get_param_model('cloudformation.CreateStack.Parameters')
        returned = self.parse_shorthand(
            p, ['ParameterKey=key,ParameterValue="foo,bar"']
        )
        expected = [{"ParameterKey": "key", "ParameterValue": "foo,bar"}]
        self.assertEqual(returned, expected)

    def test_csv_syntax_single_quoted(self):
        p = self.get_param_model('cloudformation.CreateStack.Parameters')
        returned = self.parse_shorthand(
            p, ["ParameterKey=key,ParameterValue='foo,bar'"]
        )
        expected = [{"ParameterKey": "key", "ParameterValue": "foo,bar"}]
        self.assertEqual(returned, expected)

    def test_csv_syntax_errors(self):
        p = self.get_param_model('cloudformation.CreateStack.Parameters')
        error_msg = "Error parsing parameter '--parameters'.*Expected"
        with self.assertRaisesRegex(ParamError, error_msg):
            self.parse_shorthand(
                p, ['ParameterKey=key,ParameterValue="foo,bar']
            )
        with self.assertRaisesRegex(ParamError, error_msg):
            self.parse_shorthand(
                p, ['ParameterKey=key,ParameterValue=foo,bar"']
            )
        with self.assertRaisesRegex(ParamError, error_msg):
            self.parse_shorthand(
                p, ['ParameterKey=key,ParameterValue=""foo,bar"']
            )
        with self.assertRaisesRegex(ParamError, error_msg):
            self.parse_shorthand(
                p, ['ParameterKey=key,ParameterValue="foo,bar\'']
            )


class TestParamShorthandCustomArguments(BaseArgProcessTest):
    def setUp(self):
        super(TestParamShorthandCustomArguments, self).setUp()
        self.shorthand = ParamShorthandParser()

    def test_list_structure_list_scalar_custom_arg(self):
        schema = {
            'type': 'array',
            'items': {
                'type': 'object',
                'properties': {
                    'Name': {'type': 'string'},
                    'Args': {'type': 'array', 'items': {'type': 'string'}},
                },
            },
        }
        argument_model = create_argument_model_from_schema(schema)
        cli_argument = CustomArgument('foo', argument_model=argument_model)

        expected = [
            {"Name": "foo", "Args": ["a", "k1=v1", "b"]},
            {"Name": "bar", "Args": ["baz"]},
            {"Name": "single_kv", "Args": ["key=value"]},
            {"Name": "single_v", "Args": ["value"]},
        ]

        simplified = self.shorthand(
            cli_argument,
            [
                "Name=foo,Args=[a,k1=v1,b]",
                "Name=bar,Args=baz",
                "Name=single_kv,Args=[key=value]",
                "Name=single_v,Args=[value]",
            ],
            'process-cli-arg.foo.bar',
        )

        self.assertEqual(simplified, expected)

    def test_struct_list_scalars(self):
        schema = {
            "type": "object",
            "properties": {
                "Consistent": {
                    "type": "boolean",
                },
                "Args": {"type": "array", "items": {"type": "string"}},
            },
        }
        argument_model = create_argument_model_from_schema(schema)
        cli_argument = CustomArgument('test', argument_model=argument_model)

        returned = self.shorthand(
            cli_argument,
            'Consistent=true,Args=foo1,foo2',
            'process-cli-arg.foo.bar',
        )
        self.assertEqual(
            returned, {'Consistent': True, 'Args': ['foo1', 'foo2']}
        )


class TestDocGen(BaseArgProcessTest):
    # These aren't very extensive doc tests, as we want to stay somewhat
    # flexible and allow the docs to slightly change without breaking these
    # tests.
    def setUp(self):
        super(TestDocGen, self).setUp()
        self.shorthand_documenter = ParamShorthandDocGen()
        self.service_name = 'foo'
        self.operation_name = 'bar'

    def get_generated_example_for(self, argument):
        # Returns a string containing the generated documentation.
        return self.shorthand_documenter.generate_shorthand_example(
            argument, self.service_name, self.operation_name
        )

    def assert_generated_example_is(self, argument, expected_docs):
        generated_docs = self.get_generated_example_for(argument)
        self.assertEqual(generated_docs, expected_docs)

    def assert_generated_example_contains(self, argument, expected_to_contain):
        generated_docs = self.get_generated_example_for(argument)
        self.assertIn(expected_to_contain, generated_docs)

    def test_gen_map_type_docs(self):
        argument = self.get_param_model('sqs.SetQueueAttributes.Attributes')
        expected_example_str = (
            "KeyName1=string,KeyName2=string\n\n"
            "Where valid key names are:\n"
        )
        self.assert_generated_example_contains(argument, expected_example_str)

    def test_gen_list_scalar_docs(self):
        self.service_name = 'elb'
        self.operation_name = 'register-instances-with-load-balancer'
        argument = self.get_param_model(
            'elb.RegisterInstancesWithLoadBalancer.Instances'
        )
        doc_string = '--instances InstanceId1 InstanceId2 InstanceId3'
        self.assert_generated_example_is(argument, doc_string)

    def test_flattens_marked_single_member_structure_list(self):
        argument = self.create_argument(
            {
                'Arg': {
                    'type': 'list',
                    'member': {
                        'type': 'structure',
                        'members': {'Bar': {'type': 'string'}},
                    },
                }
            },
            'arg',
        )
        argument.argument_model = argument.argument_model.members['Arg']
        uses_old_list = 'awscli.argprocess.ParamShorthand._uses_old_list_case'
        with mock.patch(uses_old_list, mock.Mock(return_value=True)):
            self.assert_generated_example_is(argument, '--arg Bar1 Bar2 Bar3')

    def test_generates_single_example_with_min_max_1(self):
        # An example of this is
        # 'workspaces rebuild-workspaces --rebuild-workspace-requests'
        argument = self.create_argument(
            {
                'Arg': {
                    'type': 'list',
                    'max': 1,
                    'min': 1,
                    'member': {
                        'type': 'structure',
                        'members': {'Bar': {'type': 'string'}},
                    },
                }
            },
            'arg',
        )
        argument.argument_model = argument.argument_model.members['Arg']
        uses_old_list = 'awscli.argprocess.ParamShorthand._uses_old_list_case'
        with mock.patch(uses_old_list, mock.Mock(return_value=True)):
            self.assert_generated_example_is(argument, '--arg Bar1')

    def test_does_not_flatten_unmarked_single_member_structure_list(self):
        argument = self.create_argument(
            {
                'Arg': {
                    'type': 'list',
                    'member': {
                        'type': 'structure',
                        'members': {'Bar': {'type': 'string'}},
                    },
                }
            },
            'arg',
        )
        argument.argument_model = argument.argument_model.members['Arg']
        uses_old_list = 'awscli.argprocess.ParamShorthand._uses_old_list_case'
        with mock.patch(uses_old_list, mock.Mock(return_value=False)):
            self.assert_generated_example_is(argument, 'Bar=string ...')

    def test_gen_list_structure_of_scalars_docs(self):
        argument = self.get_param_model('elb.CreateLoadBalancer.Listeners')
        generated_example = self.get_generated_example_for(argument)
        self.assertIn('Protocol=string', generated_example)
        self.assertIn('LoadBalancerPort=integer', generated_example)
        self.assertIn('InstanceProtocol=string', generated_example)
        self.assertIn('InstancePort=integer', generated_example)
        self.assertIn('SSLCertificateId=string', generated_example)

    def test_gen_list_structure_multiple_scalar_docs(self):
        expected = (
            'Scalar1=string,' 'Scalar2=string,' 'List1=string,string ...'
        )
        m = (
            model.DenormalizedStructureBuilder()
            .with_members(
                OrderedDict(
                    [
                        (
                            'List',
                            {
                                'type': 'list',
                                'member': {
                                    'type': 'structure',
                                    'members': OrderedDict(
                                        [
                                            ('Scalar1', {'type': 'string'}),
                                            ('Scalar2', {'type': 'string'}),
                                            (
                                                'List1',
                                                {
                                                    'type': 'list',
                                                    'member': {
                                                        'type': 'string'
                                                    },
                                                },
                                            ),
                                        ]
                                    ),
                                },
                            },
                        ),
                    ]
                )
            )
            .build_model()
            .members['List']
        )
        argument = mock.Mock()
        argument.argument_model = m
        argument.name = 'foo'
        argument.cli_name = '--foo'
        generated_example = self.get_generated_example_for(argument)
        self.assertIn(expected, generated_example)

    def test_gen_list_structure_list_scalar_scalar_docs(self):
        # Verify that we have *two* top level list items displayed,
        # so we make it clear that multiple values are separated by spaces.
        argument = self.get_param_model('ec2.DescribeInstances.Filters')
        generated_example = self.get_generated_example_for(argument)
        self.assertIn('Name=string,Values=string,string', generated_example)

    def test_gen_structure_list_scalar_docs(self):
        argument = self.create_argument(
            OrderedDict(
                [
                    ('Consistent', {'type': 'boolean'}),
                    ('Args', {'type': 'list', 'member': {'type': 'string'}}),
                ]
            ),
            'foo',
        )
        generated_example = self.get_generated_example_for(argument)
        self.assertIn(
            'Consistent=boolean,Args=string,string', generated_example
        )

    def test_can_gen_recursive_structure(self):
        argument = self.get_param_model('dynamodb.PutItem.Item')
        generated_example = self.get_generated_example_for(argument)

    def test_can_document_nested_structs(self):
        argument = self.get_param_model('ec2.RunInstances.BlockDeviceMappings')
        generated_example = self.get_generated_example_for(argument)
        self.assertRegex(generated_example, r'Ebs={\w+=\w+')

    def test_can_document_nested_lists(self):
        argument = self.create_argument(
            {
                'A': {
                    'type': 'list',
                    'member': {
                        'type': 'list',
                        'member': {'type': 'string'},
                    },
                },
            }
        )
        generated_example = self.get_generated_example_for(argument)
        self.assertIn('A=[[string,string],[string,string]]', generated_example)

    def test_can_generated_nested_maps(self):
        argument = self.create_argument(
            {
                'A': {
                    'type': 'map',
                    'key': {'type': 'string'},
                    'value': {'type': 'string'},
                },
            }
        )
        generated_example = self.get_generated_example_for(argument)
        self.assertIn('A={KeyName1=string,KeyName2=string}', generated_example)

    def test_list_of_structures_with_triple_dots(self):
        list_shape = {
            'type': 'list',
            'member': {'shape': 'StructShape'},
        }
        shapes = {
            'Top': list_shape,
            'String': {'type': 'string'},
            'StructShape': {
                'type': 'structure',
                'members': OrderedDict(
                    [
                        ('A', {'shape': 'String'}),
                        ('B', {'shape': 'String'}),
                    ]
                ),
            },
        }
        m = model.ListShape(
            shape_name='Top',
            shape_model=list_shape,
            shape_resolver=model.ShapeResolver(shapes),
        )
        argument = mock.Mock()
        argument.argument_model = m
        argument.name = 'foo'
        argument.cli_name = '--foo'
        generated_example = self.get_generated_example_for(argument)
        self.assertIn('A=string,B=string ...', generated_example)

    def test_handle_special_case_value_struct_not_documented(self):
        argument = self.create_argument({'Value': {'type': 'string'}})
        generated_example = self.get_generated_example_for(argument)
        # This is one of the special cases, we shouldn't generate any
        # shorthand example for this shape.
        self.assertIsNone(generated_example)

    def test_can_document_recursive_struct(self):
        # It's a little more work to set up a recursive
        # shape because DenormalizedStructureBuilder cannot handle
        # recursion.
        struct_shape = {
            'type': 'structure',
            'members': OrderedDict(
                [
                    ('Recurse', {'shape': 'SubShape'}),
                    ('Scalar', {'shape': 'String'}),
                ]
            ),
        }
        shapes = {
            'Top': struct_shape,
            'String': {'type': 'string'},
            'SubShape': {
                'type': 'structure',
                'members': OrderedDict(
                    [
                        ('SubRecurse', {'shape': 'Top'}),
                        ('Scalar', {'shape': 'String'}),
                    ]
                ),
            },
        }
        m = model.StructureShape(
            shape_name='Top',
            shape_model=struct_shape,
            shape_resolver=model.ShapeResolver(shapes),
        )
        argument = mock.Mock()
        argument.argument_model = m
        argument.name = 'foo'
        argument.cli_name = '--foo'
        generated_example = self.get_generated_example_for(argument)
        self.assertIn(
            'Recurse={SubRecurse={( ... recursive ... ),Scalar=string},'
            'Scalar=string},Scalar=string',
            generated_example,
        )

    def test_skip_deeply_nested_shorthand(self):
        # The eventual goal is to have a better way to document
        # deeply nested shorthand params, but for now, we'll
        # only document shorthand params up to a certain stack level.
        argument = self.create_argument(
            {
                'A': {
                    'type': 'structure',
                    'members': {
                        'B': {
                            'type': 'structure',
                            'members': {
                                'C': {
                                    'type': 'structure',
                                    'members': {
                                        'D': {'type': 'string'},
                                    },
                                }
                            },
                        }
                    },
                },
            }
        )
        generated_example = self.get_generated_example_for(argument)
        self.assertEqual(generated_example, '')

    def test_structure_within_map(self):
        argument = self.create_argument(
            {
                'A': {
                    'type': 'map',
                    'key': {'type': 'string'},
                    'value': {
                        'type': 'structure',
                        'members': {
                            'B': {'type': 'string'},
                        },
                    },
                },
            }
        )
        generated_example = self.get_generated_example_for(argument)
        self.assertEqual('A={KeyName1={B=string},KeyName2={B=string}}', generated_example)


class TestUnpackJSONParams(BaseArgProcessTest):
    def setUp(self):
        super(TestUnpackJSONParams, self).setUp()
        self.simplify = ParamShorthandParser()

    def test_json_with_spaces(self):
        p = self.get_param_model('ec2.RunInstances.BlockDeviceMappings')
        # If a user specifies the json with spaces, it will show up as
        # a multi element list.  For example:
        # --block-device-mappings [{ "DeviceName":"/dev/sdf",
        # "VirtualName":"ephemeral0"}, {"DeviceName":"/dev/sdg",
        # "VirtualName":"ephemeral1" }]
        #
        # Will show up as:
        block_device_mapping = [
            '[{',
            'DeviceName:/dev/sdf,',
            'VirtualName:ephemeral0},',
            '{DeviceName:/dev/sdg,',
            'VirtualName:ephemeral1',
            '}]',
        ]
        # The shell has removed the double quotes so this is invalid
        # JSON, but we should still raise a better exception.
        with self.assertRaises(ParamError) as e:
            unpack_cli_arg(p, block_device_mapping)
        # Parameter name should be in error message.
        self.assertIn('--block-device-mappings', str(e.exception))
        # The actual JSON itself should be in the error message.
        # Becaues this is a list, only the first element in the JSON
        # will show.  This will at least let customers know what
        # we tried to parse.
        self.assertIn('[{', str(e.exception))


class TestJSONValueHeaderParams(BaseArgProcessTest):
    def setUp(self):
        super(TestJSONValueHeaderParams, self).setUp()
        self.p = self.get_param_model(
            'lex-runtime.PostContent.sessionAttributes'
        )

    def test_json_value_dict(self):
        value = '{"foo": "bar"}'
        self.assertEqual(
            unpack_cli_arg(self.p, value), OrderedDict([('foo', 'bar')])
        )

    def test_json_value_list(self):
        value = '["foo", "bar"]'
        self.assertEqual(unpack_cli_arg(self.p, value), ['foo', 'bar'])

    def test_json_value_int(self):
        value = "5"
        self.assertEqual(unpack_cli_arg(self.p, value), 5)

    def test_json_value_float(self):
        value = "1.2"
        self.assertEqual(unpack_cli_arg(self.p, value), 1.2)

    def test_json_value_string(self):
        value = '"5"'
        self.assertEqual(unpack_cli_arg(self.p, value), '5')

    def test_json_value_boolean(self):
        value = "true"
        self.assertEqual(unpack_cli_arg(self.p, value), True)
        value = "false"
        self.assertEqual(unpack_cli_arg(self.p, value), False)

    def test_json_value_null(self):
        value = 'null'
        self.assertEqual(unpack_cli_arg(self.p, value), None)

    def test_json_value_decode_error(self):
        value = 'invalid string to be serialized'
        with self.assertRaises(ParamError):
            unpack_cli_arg(self.p, value)


class TestArgumentPercentEscaping(BaseArgProcessTest):
    def _test_percent_escaping(self, arg_type, arg_class, doc_string):
        argument = self.create_argument(
            {
                'Test': {
                    'type': arg_type,
                    'documentation': doc_string,
                }
            }
        )
        arg = arg_class(
            'test-arg',
            argument.argument_model.members['Test'],
            mock.Mock(),
            mock.Mock(),
            is_required=False,
        )
        arg_table = {arg.name: arg}
        parser = ArgTableArgParser(arg_table)
        help_output = parser.format_help()
        self.assertIn(doc_string, help_output)

    def test_cli_argument_escapes_percent(self):
        self._test_percent_escaping('string', CLIArgument, 'Symbols: % ^ & *')

    def test_boolean_argument_escapes_percent(self):
        self._test_percent_escaping('boolean', BooleanArgument, 'Symbols: % ^ & *')

    def test_cli_argument_escapes_url_encoded_percent(self):
        self._test_percent_escaping('string', CLIArgument, 'File: test%28file%29.png')

    def test_boolean_argument_escapes_url_encoded_percent(self):
        self._test_percent_escaping('boolean', BooleanArgument, 'File: test%28file%29.png')

if __name__ == '__main__':
    unittest.main()



# Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import base64
import json
import os
from datetime import datetime

from awscli.compat import urlparse
from awscli.testutils import BaseAWSCommandParamsTest, mock


class TestGetTokenCommand(BaseAWSCommandParamsTest):
    def setUp(self):
        super(TestGetTokenCommand, self).setUp()
        self.cluster_name = 'MyCluster'
        self.role_arn = 'arn:aws:iam::012345678910:role/RoleArn'
        self.access_key = 'ABCDEFGHIJKLMNOPQRST'
        self.secret_key = 'TSRQPONMLKJUHGFEDCBA'
        self.session_token = 'TOKENTOKENTOKENTOKEN'
        self.environ['AWS_ACCESS_KEY_ID'] = self.access_key
        self.environ['AWS_SECRET_ACCESS_KEY'] = self.secret_key
        self.expected_token_prefix = 'k8s-aws-v1.'

    def run_get_token(self, cmd):
        response, _, _ = self.run_cmd(cmd)
        return json.loads(response)

    def assert_url_correct(
        self,
        response,
        expected_endpoint='sts.us-east-1.amazonaws.com',
        expected_signing_region='us-east-1',
        has_session_token=False,
    ):
        url = self._get_url(response)
        url_components = urlparse.urlparse(url)
        self.assertEqual(url_components.netloc, expected_endpoint)
        parsed_qs = urlparse.parse_qs(url_components.query)
        self.assertIn(
            expected_signing_region, parsed_qs['X-Amz-Credential'][0]
        )
        if has_session_token:
            self.assertEqual(
                [self.session_token], parsed_qs['X-Amz-Security-Token']
            )
        else:
            self.assertNotIn('X-Amz-Security-Token', parsed_qs)

        self.assertIn(self.access_key, parsed_qs['X-Amz-Credential'][0])
        self.assertIn('x-k8s-aws-id', parsed_qs['X-Amz-SignedHeaders'][0])

    def _get_url(self, response):
        token = response['status']['token']
        b64_token = token[len(self.expected_token_prefix) :].encode()
        missing_padding = len(b64_token) % 4
        if missing_padding:
            b64_token += b'=' * (4 - missing_padding)
        return base64.urlsafe_b64decode(b64_token).decode()

    def set_kubernetes_exec_info(self, api_version):
        # Set KUBERNETES_EXEC_INFO env var with default payload and provided
        # api version
        self.environ['KUBERNETES_EXEC_INFO'] = (
            '{"kind":"ExecCredential",'
            f'"apiVersion":"client.authentication.k8s.io/{api_version}",'
            '"spec":{"interactive":true}}'
        )

    @mock.patch('awscli.customizations.eks.get_token.datetime')
    def test_get_token(self, mock_datetime):
        mock_datetime.utcnow.return_value = datetime(2019, 10, 23, 23, 0, 0, 0)
        cmd = 'eks get-token --cluster-name %s' % self.cluster_name
        response = self.run_get_token(cmd)
        self.assertEqual(
            response,
            {
                "kind": "ExecCredential",
                "apiVersion": "client.authentication.k8s.io/v1beta1",
                "spec": {},
                "status": {
                    "expirationTimestamp": "2019-10-23T23:14:00Z",
                    "token": mock.ANY,  # This is asserted in later cases
                },
            },
        )

    @mock.patch('awscli.customizations.eks.get_token.datetime')
    def test_query_nested_object(self, mock_datetime):
        mock_datetime.utcnow.return_value = datetime(2019, 10, 23, 23, 0, 0, 0)
        cmd = 'eks get-token --cluster-name %s' % self.cluster_name
        cmd += ' --query status'
        response = self.run_get_token(cmd)
        self.assertEqual(
            response,
            {
                "expirationTimestamp": "2019-10-23T23:14:00Z",
                "token": mock.ANY,  # This is asserted in later cases
            },
        )

    def test_query_value(self):
        cmd = 'eks get-token --cluster-name %s' % self.cluster_name
        cmd += ' --query apiVersion'
        response = self.run_get_token(cmd)
        self.assertEqual(
            response,
            "client.authentication.k8s.io/v1beta1",
        )

    @mock.patch('awscli.customizations.eks.get_token.datetime')
    def test_output_text(self, mock_datetime):
        mock_datetime.utcnow.return_value = datetime(2019, 10, 23, 23, 0, 0, 0)
        cmd = 'eks get-token --cluster-name %s' % self.cluster_name
        cmd += ' --output text'
        stdout, _, _ = self.run_cmd(cmd)
        self.assertIn("ExecCredential", stdout)
        self.assertIn("client.authentication.k8s.io/v1beta1", stdout)
        self.assertIn("2019-10-23T23:14:00Z", stdout)

    @mock.patch('awscli.customizations.eks.get_token.datetime')
    def test_output_table(self, mock_datetime):
        mock_datetime.utcnow.return_value = datetime(2019, 10, 23, 23, 0, 0, 0)
        cmd = 'eks get-token --cluster-name %s' % self.cluster_name
        cmd += ' --output table'
        stdout, _, _ = self.run_cmd(cmd)
        self.assertIn("ExecCredential", stdout)
        self.assertIn("client.authentication.k8s.io/v1beta1", stdout)
        self.assertIn("2019-10-23T23:14:00Z", stdout)

    def test_url(self):
        cmd = 'eks get-token --cluster-name %s' % self.cluster_name
        response = self.run_get_token(cmd)
        self.assert_url_correct(response)

    def test_url_with_region(self):
        cmd = 'eks get-token --cluster-name %s' % self.cluster_name
        cmd += ' --region us-west-2'
        response = self.run_get_token(cmd)
        self.assert_url_correct(
            response,
            expected_endpoint='sts.us-west-2.amazonaws.com',
            expected_signing_region='us-west-2',
        )

    def test_url_with_arn(self):
        cmd = 'eks get-token --cluster-name %s' % self.cluster_name
        cmd += ' --role-arn %s' % self.role_arn
        self.parsed_responses = [
            {
                "Credentials": {
                    "AccessKeyId": self.access_key,
                    "SecretAccessKey": self.secret_key,
                    "SessionToken": self.session_token,
                },
            }
        ]
        response = self.run_get_token(cmd)
        assume_role_call = self.operations_called[0]
        self.assertEqual(assume_role_call[0].name, 'AssumeRole')
        self.assertEqual(
            assume_role_call[1],
            {'RoleArn': self.role_arn, 'RoleSessionName': 'EKSGetTokenAuth'},
        )
        self.assert_url_correct(response, has_session_token=True)

    def test_token_has_no_padding(self):
        cmd = 'eks get-token --cluster-name %s' % self.cluster_name
        num_rounds = 100
        # It is difficult to patch everything out to get an exact
        # reproduceable token. So to make sure there is no padding, we
        # run the command N times and make sure there is no padding in the
        # generated token.
        for _ in range(num_rounds):
            response = self.run_get_token(cmd)
            self.assertNotIn('=', response['status']['token'])

    def test_url_different_partition(self):
        cmd = 'eks get-token --cluster-name %s' % self.cluster_name
        cmd += ' --region cn-north-1'
        response = self.run_get_token(cmd)
        self.assert_url_correct(
            response,
            expected_endpoint='sts.cn-north-1.amazonaws.com.cn',
            expected_signing_region='cn-north-1',
        )

    def test_api_version_discovery_deprecated(self):
        self.set_kubernetes_exec_info('v1alpha1')
        cmd = 'eks get-token --cluster-name %s' % self.cluster_name
        stdout, stderr, _ = self.run_cmd(cmd)
        response = json.loads(stdout)

        self.assertEqual(
            response["apiVersion"],
            "client.authentication.k8s.io/v1alpha1",
        )

        self.assertEqual(
            stderr,
            (
                "Kubeconfig user entry is using deprecated API "
                "version client.authentication.k8s.io/v1alpha1. Run 'aws eks update-kubeconfig' to update.\n"
            ),
        )

    def test_api_version_discovery_malformed(self):
        self.environ['KUBERNETES_EXEC_INFO'] = (
            '{{"kind":"ExecCredential",'
            '"apiVersion":"client.authentication.k8s.io/v1alpha1",'
            '"spec":{"interactive":true}}'
        )
        cmd = 'eks get-token --cluster-name %s' % self.cluster_name
        stdout, stderr, _ = self.run_cmd(cmd)
        response = json.loads(stdout)

        self.assertEqual(
            response["apiVersion"],
            "client.authentication.k8s.io/v1beta1",
        )

        self.assertEqual(
            stderr,
            (
                "Error parsing KUBERNETES_EXEC_INFO, defaulting to client.authentication.k8s.io/v1beta1. "
                "This is likely a bug in your Kubernetes client. Please update your Kubernetes client.\n"
            ),
        )

    def test_api_version_discovery_empty(self):
        cmd = 'eks get-token --cluster-name %s' % self.cluster_name
        stdout, stderr, _ = self.run_cmd(cmd)
        response = json.loads(stdout)

        self.assertEqual(
            response["apiVersion"],
            "client.authentication.k8s.io/v1beta1",
        )

        self.assertEqual(
            stderr,
            "",
        )

    def test_api_version_discovery_v1(self):
        self.set_kubernetes_exec_info('v1')
        cmd = 'eks get-token --cluster-name %s' % self.cluster_name
        stdout, stderr, _ = self.run_cmd(cmd)
        response = json.loads(stdout)

        self.assertEqual(
            response["apiVersion"],
            "client.authentication.k8s.io/v1",
        )

        self.assertEqual(stderr, "")

    def test_api_version_discovery_v1beta1(self):
        self.set_kubernetes_exec_info('v1beta1')
        cmd = 'eks get-token --cluster-name %s' % self.cluster_name
        stdout, stderr, _ = self.run_cmd(cmd)
        response = json.loads(stdout)

        self.assertEqual(
            response["apiVersion"],
            "client.authentication.k8s.io/v1beta1",
        )

        self.assertEqual(stderr, "")

    def test_api_version_discovery_unknown(self):
        self.set_kubernetes_exec_info('v2')
        cmd = 'eks get-token --cluster-name %s' % self.cluster_name
        stdout, stderr, _ = self.run_cmd(cmd)
        response = json.loads(stdout)

        self.assertEqual(
            response["apiVersion"],
            "client.authentication.k8s.io/v1beta1",
        )

        self.assertEqual(
            stderr,
            (
                "Unrecognized API version in KUBERNETES_EXEC_INFO, defaulting to client.authentication.k8s.io/v1beta1. "
                "This is likely due to an outdated AWS CLI. Please update your AWS CLI.\n"
            ),
        )



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os
import re
import shutil

import botocore

from awscli.clidriver import create_clidriver
from awscli.testutils import BaseCLIDriverTest, FileCreator, mock
from tests import RawResponse


class RegionCapture:
    def __init__(self):
        self.region = None

    def __call__(self, request, **kwargs):
        url = request.url
        region = re.match(r'https://.*?\.(.*?)\.amazonaws\.com', url).groups(
            1
        )[0]
        self.region = region


class TestSession(BaseCLIDriverTest):
    def setUp(self):
        super(TestSession, self).setUp()
        urllib3_session_send = 'botocore.httpsession.URLLib3Session.send'
        self._urllib3_patch = mock.patch(urllib3_session_send)
        self._send = self._urllib3_patch.start()
        self._send.side_effect = self.get_response
        self._responses = []

    def tearDown(self):
        self._urllib3_patch.stop()

    def get_response(self, request):
        response = self._responses.pop(0)
        if isinstance(response, Exception):
            raise response
        return response

    def add_response(self, body, status_code=200):
        response = botocore.awsrequest.AWSResponse(
            url='http://169.254.169.254/',
            status_code=status_code,
            headers={},
            raw=RawResponse(body),
        )
        self._responses.append(response)

    def test_imds_region_is_used_as_fallback_wo_v2_support(self):
        # Remove region override from the environment variables.
        self.environ.pop('AWS_DEFAULT_REGION', 0)
        # First response should be from the IMDS server for security token
        # if server supports IMDSv1 only there will be no response for token
        self.add_response(None)
        # Then another response from the IMDS server for an availability
        # zone.
        self.add_response(b'us-mars-2a')
        # Once a region is fetched form the IMDS server we need to mock an
        # XML response from ec2 so that the CLI driver doesn't throw an error
        # during parsing.
        self.add_response(b'<?xml version="1.0" ?><foo><bar>text</bar></foo>')
        capture = RegionCapture()
        self.session.register('before-send.ec2.*', capture)
        self.driver.main(['ec2', 'describe-instances'])
        self.assertEqual(capture.region, 'us-mars-2')

    def test_imds_region_is_used_as_fallback_with_v2_support(self):
        # Remove region override from the environment variables.
        self.environ.pop('AWS_DEFAULT_REGION', 0)
        # First response should be from the IMDS server for security token
        # if server supports IMDSv2 it'll return token
        self.add_response(b'token')
        # Then another response from the IMDS server for an availability
        # zone.
        self.add_response(b'us-mars-2a')
        # Once a region is fetched form the IMDS server we need to mock an
        # XML response from ec2 so that the CLI driver doesn't throw an error
        # during parsing.
        self.add_response(b'<?xml version="1.0" ?><foo><bar>text</bar></foo>')
        capture = RegionCapture()
        self.session.register('before-send.ec2.*', capture)
        self.driver.main(['ec2', 'describe-instances'])
        self.assertEqual(capture.region, 'us-mars-2')

    def test_user_agent_in_request(self):
        self.add_response(b'<?xml version="1.0" ?><foo><bar>text</bar></foo>')
        self.driver.main(['ec2', 'describe-instances'])
        self.assertTrue('User-Agent' in self._send.call_args[0][0].headers)
        self.assertTrue(
            'aws-cli'
            in self._send.call_args[0][0].headers['User-Agent'].decode('utf-8')
        )
        self.assertTrue(
            'md/installer'
            in self._send.call_args[0][0].headers['User-Agent'].decode('utf-8')
        )
        self.assertTrue(
            'md/command'
            in self._send.call_args[0][0].headers['User-Agent'].decode('utf-8')
        )


class TestPlugins(BaseCLIDriverTest):
    def setUp(self):
        super(TestPlugins, self).setUp()
        self.files = FileCreator()
        self.plugins_site_packages = os.path.join(
            self.files.rootdir, 'site-packages'
        )
        self.plugin_module_name = 'add_awscli_cmd_plugin'
        self.plugin_filename = (
            os.path.join(os.path.dirname(__file__), self.plugin_module_name)
            + '.py'
        )
        self.setup_plugin_site_packages()

    def setup_plugin_site_packages(self):
        os.makedirs(self.plugins_site_packages)
        shutil.copy(self.plugin_filename, self.plugins_site_packages)

    def tearDown(self):
        super(TestPlugins, self).tearDown()
        self.files.remove_all()

    def assert_plugin_loaded(self, clidriver):
        self.assertIn('plugin-test-cmd', clidriver.subcommand_table)

    def assert_plugin_not_loaded(self, clidriver):
        self.assertNotIn('plugin-test-cmd', clidriver.subcommand_table)

    def create_config(self, config_contents):
        config_file = self.files.create_file('config', config_contents)
        self.environ['AWS_CONFIG_FILE'] = config_file

    def test_plugins_loaded_from_specified_path(self):
        self.create_config(
            '[plugins]\n'
            'cli_legacy_plugin_path = %s\n'
            'myplugin = %s\n'
            % (self.plugins_site_packages, self.plugin_module_name)
        )
        clidriver = create_clidriver()
        self.assert_plugin_loaded(clidriver)

    def test_plugins_are_not_loaded_when_path_specified(self):
        self.create_config(
            '[plugins]\n' 'myplugin = %s\n' % self.plugin_module_name
        )
        clidriver = create_clidriver()
        self.assert_plugin_not_loaded(clidriver)

    def test_looks_in_all_specified_paths(self):
        nonexistent_dir = os.path.join(self.files.rootdir, 'no-exist')
        plugin_path = os.pathsep.join(
            [nonexistent_dir, self.plugins_site_packages]
        )
        self.create_config(
            '[plugins]\n'
            'cli_legacy_plugin_path = %s\n'
            'myplugin = %s\n' % (plugin_path, self.plugin_module_name)
        )
        clidriver = create_clidriver()
        self.assert_plugin_loaded(clidriver)



# Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestColor(BaseAWSCommandParamsTest):
    def test_pipe_color(self):
        command = "s3api list-buckets --output table --color on"
        self.parsed_response = {
            "Owner": {"DisplayName": "foo", "ID": "bar"},
            "Buckets": [
                {
                    "CreationDate": "2016-06-15T16:49:44.000Z",
                    "Name": "foo-bucket",
                }
            ],
        }
        stdout, stderr, rc = self.run_cmd(command, 0)
        # `\x1b` is the ANSI color prefix character.
        self.assertIn('\x1b', stdout)



# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import datetime
import json

from dateutil.tz import tzlocal

from awscli.clidriver import AWSCLIEntryPoint, create_clidriver
from awscli.testutils import BaseCLIWireResponseTest, FileCreator


class TestCLITimestampParser(BaseCLIWireResponseTest):
    def setUp(self):
        super(TestCLITimestampParser, self).setUp()
        self.files = FileCreator()
        self.wire_response = json.dumps(
            {
                'builds': [
                    {
                        'startTime': 0,
                    }
                ]
            }
        ).encode('utf-8')
        self.command = ['codebuild', 'batch-get-builds', '--ids', 'foo']
        self.patch_send(content=self.wire_response)

    def tearDown(self):
        super(TestCLITimestampParser, self).tearDown()
        self.files.remove_all()

    def test_iso(self):
        self.environ['AWS_CONFIG_FILE'] = self.files.create_file(
            'iso', '[default]\ncli_timestamp_format = iso8601\n'
        )
        self.driver = create_clidriver()
        self.entry_point = AWSCLIEntryPoint(self.driver)
        expected_time = (
            datetime.datetime.fromtimestamp(0)
            .replace(tzinfo=tzlocal())
            .isoformat()
        )

        stdout, _, _ = self.run_cmd(self.command)
        json_response = json.loads(stdout)
        start_time = json_response["builds"][0]["startTime"]
        self.assertEqual(expected_time, start_time)

    def test_none(self):
        self.environ['AWS_CONFIG_FILE'] = self.files.create_file(
            'none', '[default]\ncli_timestamp_format = wire\n'
        )
        self.driver = create_clidriver()
        self.entry_point = AWSCLIEntryPoint(self.driver)
        expected_time = 0

        stdout, _, _ = self.run_cmd(self.command)
        json_response = json.loads(stdout)
        start_time = json_response["builds"][0]["startTime"]
        self.assertEqual(expected_time, start_time)

    def test_default(self):
        self.driver = create_clidriver()
        self.entry_point = AWSCLIEntryPoint(self.driver)
        expected_time = (
            datetime.datetime.fromtimestamp(0)
            .replace(tzinfo=tzlocal())
            .isoformat()
        )

        stdout, _, _ = self.run_cmd(self.command)
        json_response = json.loads(stdout)
        start_time = json_response["builds"][0]["startTime"]
        self.assertEqual(expected_time, start_time)



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from tests.unit.customizations.emr import (
    EMRBaseAWSCommandParamsTest as BaseAWSCommandParamsTest,
)


class TestAddTags(BaseAWSCommandParamsTest):
    prefix = 'emr add-tags'

    def test_add_tags_key_value(self):
        args = ' --resource-id j-ABC123456 --tags k1=v1 k2=v2'
        cmdline = self.prefix + args
        result = {
            'ResourceId': 'j-ABC123456',
            'Tags': [
                {'Key': 'k1', 'Value': 'v1'},
                {'Key': 'k2', 'Value': 'v2'},
            ],
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_add_tags_key_with_empty_value(self):
        args = ' --resource-id j-ABC123456 --tags k1=v1 k2 k3=v3'
        cmdline = self.prefix + args
        result = {
            'ResourceId': 'j-ABC123456',
            'Tags': [
                {'Key': 'k1', 'Value': 'v1'},
                {'Key': 'k2', 'Value': ''},
                {'Key': 'k3', 'Value': 'v3'},
            ],
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_add_tags_key_value_space(self):
        cmdline = [
            'emr',
            'add-tags',
            '--resource-id',
            'j-ABC123456',
            '--tags',
            'k1=v1',
            'k2',
            'k3=v3 v4',
        ]
        result = {
            'ResourceId': 'j-ABC123456',
            'Tags': [
                {'Key': 'k1', 'Value': 'v1'},
                {'Key': 'k2', 'Value': ''},
                {'Key': 'k3', 'Value': 'v3 v4'},
            ],
        }
        self.assert_params_for_cmd(cmdline, result)


if __name__ == "__main__":
    unittest.main()



# Copyright 2021 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestStartCutover(BaseAWSCommandParamsTest):
    PREFIX = 'mgn start-cutover'
    SOURCE_SERVER_ID = 'a' * 50

    def test_accepts_old_argname(self):
        cmdline = f'{self.PREFIX} --source-server-i-ds {self.SOURCE_SERVER_ID}'
        params = {
            'sourceServerIDs': [self.SOURCE_SERVER_ID],
        }
        self.assert_params_for_cmd(cmdline, params)

    def test_accepts_fixed_param_name(self):
        cmdline = f'{self.PREFIX} --source-server-ids {self.SOURCE_SERVER_ID}'
        params = {
            'sourceServerIDs': [self.SOURCE_SERVER_ID],
        }
        self.assert_params_for_cmd(cmdline, params)
